
/**
 * AnnotatePanel.java
 *
 * A class that allows one to annotate regions of images.  Currently
 * working on the ability to draw a GeneralPath to the screen.  Other
 * milestones will follow.  
 *
 * The AnnotatePanel has the following GUI elements:
 *   AnnotatorPanel (draws regions on image)
 *   AnnotationInformationPanel (shows information for an Annotation)
 *   JToolBar (use to select what sort of annotation shape to draw)
 *
 * This class is a JApplet, but will also try to function as an
 * application.  Based heavily on the Odd_Shape.java applet from the
 * Java tutorial for 2d graphics on how to draw GeneralPath objects.
 *
 *  Changed it to just a panel, as it is now being driven by GMAOGUI.java.
 *  
 * LOTD:
 *   Fix sizes on resize event. 
 *
 * 03/07/2000 Milestone 1: 
 *   I can now draw a series of GeneralPath objects in the AnnotatorPanel.
 *
 * 03/08/2000 Milestone 2: 
 *   I can now put pictures in the background of the thing.  
 *   Draws the lines while you draw as you go.  
 *  
 * 03/09/2000 
 *   Changed the GeneralPath objects to Annotations.  
 *
 * 03/14/2000
 *   Changed Annotations and drawing to support ovals and rectangles.  
 *
 * 06/26/2000
 *   Changed to a JPanel, it is now being driven by GMAOGUI.java
 *
 * 06/26/2000 
 *   Working on adding a function that passes a DOM DocumentFragment 
 *   object that defines the Page characteristics (background, 
 *   and annotations) and throws them up.  
 *
 * 06/27/2000 
 *   We can now through up backgrounds and bubbles that have been 
 *   defined in the XML file.  The next step is to write a function to 
 *   convert the internal representation into a document fragment so we 
 *   can pass that onto the GMAOGUI for each page, letting it update the
 *   DOM.  Then we can write that out, and we are off...  
 *  
 * 07/06/2000 
 *   Made lots of changes recently.  I am now moving away from auto-loading
 *   the images with the ImageIcon interface, and am instead implementing
 *   my own MediaTracker on the AnnotatePanel class so that I can get
 *   some better information tracking and handling for the image loading.
 * 
 * 07/13/2000
 *   Biggest recent change.  Added a zoom control to the toolbar.  
 *   Zooming the image is fine, there is a bit of trouble with zooming
 *   the associated annotations.  GMAO2k will always be slower in 
 *   a scale that is not the native (100) scale because we have to 
 *   intercept the Annotations and then re-draw then in another scale.  
 *   Actually, what we do is we re-scale all of the annotations, so it
 *   is only a one time hit.  The annotations then track their native (100)
 *   size and the size at the current scale.  If they are scaled, they 
 *   return the scaled object.  
 *
 * Created: Mon Mar 06 23:46:27 2000
 *
 * @author David Evans
 * @version 0.2 alpha 
 */


/* 
   GMAO - Image Annotation for Manga Translation
   Copyright (C) 2000  David Kirk Evans
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

   David Kirk Evans (devans@cs.columbia.edu)
   http://www.cs.columbia.edu/~devans/   
*/


/* Import packages */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.geom.*;
import java.awt.image.BufferedImage;
import java.util.*;
import java.net.URL;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import java.awt.image.ImageObserver;
import java.awt.MediaTracker;
import java.text.DateFormat;

/* Import other GMAO2k stuff */
import Annotator;
import Annotation;

public class AnnotatePanel extends JPanel implements ActionListener, ImageObserver {
    JToolBar toolBar;  // Tool bar for pane - what shape do we draw? 
    AnnotatorPanel annPanel;  // Panel to annotate in
    JFrame annInfoFrame; // Frame for annotation information
    AnnotationInformationPanel annInfoPanel; // Where we put annotation info
    ImageIcon theBackground; // Current background we are using
    Annotator currentAnnotator;
    static JLabel infoDisplay;  // A JLabel that we can put status info in.
    Hashtable imageCache;  // A hashtable to track the MediaTracker IDs for each image name
    int currentImageWidth=-1;
    int currentImageHeight=-1; // Used to get a % while we are loading...
    String currentFileName=""; // Used to track name of current image file
    int currentImageScale=100;  // %.  What scale to view curernt image at. 
    String[] zoom = {"200%", "150%", "125%", "100%", "75%", "50%", "25%", "Fit Window"};
    JComboBox zoomLevel;  // The combo box 
    JScrollPane scrollPane; // Scrollpane for annPanel.  Need for zoomLevel...
    GMAOCacheWatcher cw;

    public AnnotatePanel(Properties props) {
	imageCache = new Hashtable();
	// Set up a watcher thread for the image cache
	cw = new GMAOCacheWatcher(imageCache, props);
	cw.start();

	// Set up zoomLevel JComboBox

	//  We want to add a JComboBox so we can change the zoom level 
	//  of the image.  We want it to be editable.  Do that later.
	zoomLevel = new JComboBox(zoom);
	//  Make it an editable combo box
	//  Give it a ComboBoxEditor that only accepts numbers
	PercentComboBoxEditor pcbe = new PercentComboBoxEditor();
	zoomLevel.setEditor(pcbe);
	zoomLevel.setEditable(true);
	zoomLevel.addActionListener(this);
	zoomLevel.setSelectedItem(zoom[3]);
	// Build the UI first.  Don't do it again!  Null background initially
	buildUI(this, null, props);
    }

    /**
     * AnnotatePanel(Properties props, DocumentFragment pageInfo)
     *
     *  Create a new AnnotatePAnel using the properties specified in 
     *  props and the page information in pageInfo.  pageInfo is a 
     *  documentFragment that is a Page element - which has as an
     *  attribute the name of the image to use as the background. 
     *  It also has Bubble elements that specify locations for 
     *  Annotations, and Translations objects that are specific 
     *  notes in the annotation.  
     */
    public AnnotatePanel(Properties props, org.w3c.dom.DocumentFragment pageInfo) {
	this(props);
	displayNewPage(props, pageInfo);
    }

    public void setProperties(Properties props) {
	// Send on down
	annPanel.setProperties(props);
	cw.setCacheSize(Integer.parseInt(props.getProperty("cacheSize")));
    }

    /**
     * displayNewPage
     *  
     *  This is the heart of the XML processing.  Here we take a 
     *  DocumentFragment that is the subtree for a given "Page" element.
     *  Then we convert all of the entities in the tree into our own 
     *  internal representation of Annotations and Notes.  That throws
     *  them up on the screen.  Yay!
     */
    public void displayNewPage(Properties props, org.w3c.dom.DocumentFragment pageInfo) {
	// System.out.println("About to display page: "+pageInfo.getFirstChild());
	if (pageInfo == null) {
	    // System.out.println("PageInfo is null");
	} else if (pageInfo.getChildNodes() == null) {
	    // System.out.println("PageInfo.getChildNodes() is null"); 
	}
	// First, get the location of the image to load
	if (pageInfo.getChildNodes().getLength() != 1) {
	    System.out.println("pageInfo document fragment has too many children!!");
	}
	org.w3c.dom.Node page = pageInfo.getFirstChild();
	// page should now be a Node of element type page.  If not,
	// something is wrong with the pageInfo DocumentFragment.  
	// Get the name of the image to load
	String imgName;
	org.w3c.dom.NamedNodeMap attrs = page.getAttributes();
	if (attrs.getNamedItem("Image") != null) {
	    imgName = attrs.getNamedItem("Image").getNodeValue();
	} else {
	    imgName = null;
	}
	if (imgName == null) {
	    System.out.println("Image attribute is null for pageInfo node!");
	}
	//  We have the name of the image to load.  Now we have to decide
	//  whether to load it locally or over a http connection.  Make
	//  this decision based on the info in the properties.
	theBackground = null;
	/*  Attempt to load the image in using either http or local file
	 *  system.  
	 */
	loadImage(props, imgName);
	/* We have now loaded the image into theBackground 
	 * Now we have to update the panel that contains it...
	 */
	annPanel.setBackground(theBackground);
	/* Now we have to create Annotation objects for each of the 
	 * bubble elements and add them to the annotations in annPanel.
	 */
	// First, find "bubble" elements -- should be children of page
	org.w3c.dom.NodeList bubbles = ((org.w3c.dom.Element)page).getElementsByTagName("Bubble");
	for (int i=0; i<bubbles.getLength(); i++) {
	    // Ensure that this child is a bubble
	    org.w3c.dom.Node bubble = bubbles.item(i);
	    if (bubble == null) {
		// System.out.println("Bubble "+i+"is null");
	    } else {
		// System.out.println("Bubble "+i+" is not null");
	    }
	    if (bubble.getNodeName().equals("Bubble")) {
		// Get Attributes.  Create Annotation for it.  
		attrs = bubble.getAttributes();
		String type=null;
		if (attrs.getNamedItem("Type") != null) {
		    type = attrs.getNamedItem("Type").getNodeValue();
		}
		Annotation note = null;
		Shape shape = null;
		String character = "unknown";
		if (attrs.getNamedItem("Character") != null) {
		    character = attrs.getNamedItem("Character").getNodeValue();
		}
		if (type.equals("Rectangle") || type.equals("Oval")) {
		    // We have two point objects.  Upper left, lower right.
		    // Get the points
		    org.w3c.dom.NodeList points = ((org.w3c.dom.Element)bubble).getElementsByTagName("Point");
		    double x1, y1, x2, y2;
		    // First two children should be the points
		    org.w3c.dom.Node point = points.item(0);
		    attrs = point.getAttributes();
		    x1 = Double.parseDouble(attrs.getNamedItem("X").getNodeValue());
		    y1 = Double.parseDouble(attrs.getNamedItem("Y").getNodeValue());
		    point = points.item(1);
		    attrs = point.getAttributes();
		    x2 = Double.parseDouble(attrs.getNamedItem("X").getNodeValue());
		    y2 = Double.parseDouble(attrs.getNamedItem("Y").getNodeValue());
		    // Now we have the points.  Make the new shape.
		    if (type.equals("Rectangle")) {
			shape = new Rectangle2D.Double(x1, y1, x2-x1, y2-y1);
		    } else if (type.equals("Oval")) {
			shape = new Ellipse2D.Double(x1, y1, x2-x1, y2-y1);
		    }
		} else if (type.equals("Polygon")) {
		    // We have some number of point objects...
		    org.w3c.dom.NodeList points = ((org.w3c.dom.Element)bubble).getElementsByTagName("Point");
		    int j=0;
		    GeneralPath poly = new GeneralPath();		    
		    while (j < points.getLength() && points.item(j).getNodeName().equals("Point")) {
			attrs = points.item(j).getAttributes();
			int x, y;
			x = Integer.parseInt(attrs.getNamedItem("X").getNodeValue());
			y = Integer.parseInt(attrs.getNamedItem("Y").getNodeValue());
			// System.out.println("Added poly point "+j+" ("+x+", "+y+")");
			if (j == 0) 
			    poly.moveTo(x,y);
			else 
			    poly.lineTo(x,y);
			j++;
		    }
		    poly.closePath();
		    shape = poly;
		}
		//  Now create annotation - we have shape in shape...
		note = new Annotation(shape);
		//  Now run through all translations and add them as notes
		org.w3c.dom.NodeList translations = ((org.w3c.dom.Element)bubble).getElementsByTagName("Translation");
		int noteNum = 1;
		for (int k=0; k<translations.getLength(); k++) {
		    if (translations.item(k).getNodeName().equals("Translation")) {
			// We have a translation.  Has at least two attributes:
			// Language and author.  Take the author as 
			// the annotator.  Put language in a custom field.
			attrs = translations.item(k).getAttributes();
			NoteCustomField langField = new NoteCustomField("Language", attrs.getNamedItem("Language").getNodeValue(), true);
			Vector lfv = new Vector();
			lfv.add(langField);
			// The first child should be a text node with the text.
			String transText = translations.item(k).getFirstChild().getNodeValue();
			Note theNote = new Note(transText, noteNum++, lfv);
			// Check if we have a date on the attrs
			if (attrs.getNamedItem("Created") != null) {
			    // Parse the date and set it.
			    try {
				theNote.setDate(DateFormat.getDateInstance().parse(attrs.getNamedItem("Created").getNodeValue()));
			    } catch (java.text.ParseException e) {
				System.out.println("Unable to parse date: "+ attrs.getNamedItem("Created").getNodeValue() + " Exception: "+e);
			    }
			}
			/* NOTE: We have to change this so that we really
			   track the different annotators in the XML file.
			   We shouldn't create a new Annotator instance each
			   time!!
			*/
			theNote.setAnnotator(new Annotator(attrs.getNamedItem("Author").getNodeValue(), ""));
			theNote.setCustomFieldValue("Character", character);
			note.addNote(theNote);
		    } 
		}
		//  Now the Annotation note has all the bubbles and so on.  
		//  Add it to the panel for drawing.  
		annPanel.addAnnotation(note);
	    } else {
		// System.out.println(bubble.getNodeName()+" was not a bubble. (" + bubble +") # "+i);
	    }
	} // For loop 
	// Tell ourself to re-paint us.  Because we changed.  A lot.
	this.invalidate();
    }

    /**
     * DocumentFragment createDOM(Document doc)
     * 
     *  This method will create a DocumentFragment that represents the
     *  current state of the displayed page, created using the document doc.
     *  Also, make sure that we create the bubbles at their native scale!
     */

    org.w3c.dom.DocumentFragment createDom(org.w3c.dom.Document doc) {
	infoDisplay.setText("Creating DOM for this page...");
	//  Set the scale to native scale
	int tempScale = currentImageScale;
	annPanel.scaleAnnotations(100);
	//  First we have to create a Document to get these things from.
	
	try {
	    org.w3c.dom.DocumentFragment frag = doc.createDocumentFragment();
	    //  First, we need to get the annotations from the annPanel
	    Vector annotations = annPanel.getAnnotations();
	    //  For each annotation we need to create a Bubble node
	    //  and the appropriate translations nodes.  
	    if (annotations == null) {
		System.out.println("There are no annotations on this page!");
		return null;
	    }
	    for (int i=0; i<annotations.size(); i++) {
		// System.out.println("Creating bubble "+i);
		Annotation note = (Annotation) annotations.elementAt(i);
		// Create a Bubble node for this Annotation
		org.w3c.dom.Element bubble = doc.createElement("Bubble");
		// Now we have to add the correct attributes to it. 
		//  We should add something to allow for the Character attr.
		bubble.setAttribute("Number", ""+i);
		if (note.getShape() instanceof Rectangle2D.Double) {
		    bubble.setAttribute("Type", "Rectangle");
		    //  Add in the points we need too.  
		    org.w3c.dom.Element point = doc.createElement("Point");
		    Rectangle2D.Double rect = 
			(Rectangle2D.Double) note.getShape();		    
		    point.setAttribute("X", ""+(int)rect.getX());
		    point.setAttribute("Y", ""+(int)rect.getY());
		    bubble.appendChild(point);
		    point = doc.createElement("Point");
		    point.setAttribute("X", ""+(int)(rect.getX() + rect.getWidth()));
		    point.setAttribute("Y", ""+(int)(rect.getY() + rect.getHeight()));
		    bubble.appendChild(point);
		} else if (note.getShape() instanceof Ellipse2D.Double) {
		    bubble.setAttribute("Type", "Oval");
		    //  Add in the points we need too.  
		    org.w3c.dom.Element point = doc.createElement("Point");
		    Ellipse2D.Double rect = 
			(Ellipse2D.Double) note.getShape();
		    point.setAttribute("X", ""+(int)rect.getX());
		    point.setAttribute("Y", ""+(int)rect.getY());
		    bubble.appendChild(point);
		    point = doc.createElement("Point");
		    point.setAttribute("X", ""+(int)(rect.getX() + rect.getWidth()));
		    point.setAttribute("Y", ""+(int)(rect.getY() + rect.getHeight()));
		    bubble.appendChild(point);
		} else if (note.getShape() instanceof GeneralPath) {
		    bubble.setAttribute("Type", "Polygon");
		    GeneralPath poly = (GeneralPath) note.getShape();
		    //  Add in the points that we need to
		    org.w3c.dom.Element point;
		    //  Get a path iterator
		    PathIterator pi = poly.getPathIterator(null);
		    double[] arr = new double[6];
		    while (!pi.isDone()) {
			point = doc.createElement("Point");
			int x,y;
			pi.currentSegment(arr);
			x = (int) arr[0];
			y = (int) arr[1];
			point.setAttribute("X", ""+x);
			point.setAttribute("Y", ""+y);
			bubble.appendChild(point);
			// Get next segment along iterator
			pi.next();
		    }
		}
		// Now bubble has all of the points in it.  
		// Check to see if the first translation has a character
		// If so, make it the character for the whole bubble
		if (note.getNumNotes() > 0) {
		    Note temp = note.getNote(0);
		    bubble.setAttribute("Character", temp.getCustomFieldValue("Character"));
		}
		// Add in the translations
	        for (int k=0; k<note.getNumNotes(); k++) {
		    Note trans = note.getNote(k);
		    // For each translation, add Language attr if there,
		    // and Author attr 
		    org.w3c.dom.Element translation = 
			doc.createElement("Translation");
		    String auth = "unknown";
		    if (trans.getAnnotator() != null) {
			auth = trans.getAnnotator().getID();
		    }
		    translation.setAttribute("Author", auth);
		    // Check for a language custom field
		    if (trans.getCustomFieldValue("Language") != null) {
			translation.setAttribute("Language", trans.getCustomFieldValue("Language"));
		    }
		    // Add in the date
		    translation.setAttribute("Created", DateFormat.getDateInstance().format(trans.getDate()));
		    // Add a text node with the actual text
		    translation.appendChild(doc.createTextNode(trans.getNote()));		    
		    // Add in the translation to the bubble
		    bubble.appendChild(translation);
		}
		// Now bubble has all the translations in it.  
		// Add the bubble to the doc fragment
		// System.out.println("Appending bubble "+bubble);
		frag.appendChild(bubble);
	    }
	    // Re-scale annotations
	    annPanel.scaleAnnotations(tempScale);
	    // Return the fragment
	    infoDisplay.setText("Created DOM!");
	    return frag;
	} catch (Exception e) {
	    System.out.println("Error trying to create a document: "+e);
	    e.printStackTrace();
	}
	// Re-scale annotations
	annPanel.scaleAnnotations(tempScale);
	// Only get here in an error case
	System.out.println("Wasn't able to create the document fragment for the page! Returning null...");
	return null;
    }

    /**
     * DocumentFragment createDOMChanges(Document doc)
     * 
     *  This method will create a DocumentFragment that represents the
     *  current set of changes made to the displayed page since it was
     *  loaded, created using the document doc.  Also, make sure that
     *  we create the bubbles at their native scale!
     *
     *  Essentially the same as createDOM(Document doc) except this uses
     *  the Vector changes
     *
     *  Also, clear out the changes when we have finished.  We assume
     *  that ones someone calls this function, they will take the dom of
     *  changes and use it to update whatever it is that needs updating. 
     *  When we get called next, we will only return the changes that 
     *  have been made since we created the last changed dom.  yo.
     */

    org.w3c.dom.DocumentFragment createDomChanges(org.w3c.dom.Document doc) {
	infoDisplay.setText("Creating DOM changes for this page...");
	//  Set the scale to native scale
	int tempScale = currentImageScale;
	annPanel.scaleAnnotations(100);
	//  First we have to create a Document to get these things from.
	
	try {
	    org.w3c.dom.DocumentFragment frag = doc.createDocumentFragment();
	    //  First, we need to get the annotations from the annPanel
	    Vector annotations = annPanel.getChanges();
	    //  For each annotation we need to create a Bubble node
	    //  and the appropriate translations nodes.  
	    if (annotations == null || annotations.size() == 0) {
		System.out.println("No changes on this page to send");
		return null;
	    }
	    for (int i=0; i<annotations.size(); i++) {
		// System.out.println("Creating bubble "+i);
		Annotation note = (Annotation) annotations.elementAt(i);
		// Create a Bubble node for this Annotation
		org.w3c.dom.Element bubble = doc.createElement("Bubble");
		// Now we have to add the correct attributes to it. 
		//  We should add something to allow for the Character attr.
		bubble.setAttribute("Number", ""+i);
		if (note.getShape() instanceof Rectangle2D.Double) {
		    bubble.setAttribute("Type", "Rectangle");
		    //  Add in the points we need too.  
		    org.w3c.dom.Element point = doc.createElement("Point");
		    Rectangle2D.Double rect = 
			(Rectangle2D.Double) note.getShape();    
		    point.setAttribute("X", ""+(int)rect.getX());
		    point.setAttribute("Y", ""+(int)rect.getY());
		    bubble.appendChild(point);
		    point = doc.createElement("Point");
		    point.setAttribute("X", ""+(int)(rect.getX() + rect.getWidth()));
		    point.setAttribute("Y", ""+(int)(rect.getY() + rect.getHeight()));
		    bubble.appendChild(point);
		} else if (note.getShape() instanceof Ellipse2D.Double) {
		    bubble.setAttribute("Type", "Oval");
		    //  Add in the points we need too.  
		    org.w3c.dom.Element point = doc.createElement("Point");
		    Ellipse2D.Double rect = 
			(Ellipse2D.Double) note.getShape();
		    point.setAttribute("X", ""+(int)rect.getX());
		    point.setAttribute("Y", ""+(int)rect.getY());
		    bubble.appendChild(point);
		    point = doc.createElement("Point");
		    point.setAttribute("X", ""+(int)(rect.getX() + rect.getWidth()));
		    point.setAttribute("Y", ""+(int)(rect.getY() + rect.getHeight()));
		    bubble.appendChild(point);
		} else if (note.getShape() instanceof GeneralPath) {
		    bubble.setAttribute("Type", "Polygon");
		    GeneralPath poly = (GeneralPath) note.getShape();
		    //  Add in the points that we need to
		    org.w3c.dom.Element point;
		    //  Get a path iterator
		    PathIterator pi = poly.getPathIterator(null);
		    double[] arr = new double[6];
		    while (!pi.isDone()) {
			point = doc.createElement("Point");
			int x,y;
			pi.currentSegment(arr);
			x = (int) arr[0];
			y = (int) arr[1];
			point.setAttribute("X", ""+x);
			point.setAttribute("Y", ""+y);
			bubble.appendChild(point);
			// Get next segment along iterator
			pi.next();
		    }
		}
		// Now bubble has all of the points in it.  
		// Check to see if the first translation has a character
		// If so, make it the character for the whole bubble
		if (note.getNumNotes() > 0) {
		    Note temp = note.getNote(0);
		    bubble.setAttribute("Character", temp.getCustomFieldValue("Character"));
		}
		// Add in the translations
	        for (int k=0; k<note.getNumNotes(); k++) {
		    Note trans = note.getNote(k);
		    // For each translation, add Language attr if there,
		    // and Author attr 
		    org.w3c.dom.Element translation = 
			doc.createElement("Translation");
		    String auth = "unknown";
		    if (trans.getAnnotator() != null) {
			auth = trans.getAnnotator().getID();
		    }
		    translation.setAttribute("Author", auth);
		    // Check for a language custom field
		    if (trans.getCustomFieldValue("Language") != null) {
			translation.setAttribute("Language", trans.getCustomFieldValue("Language"));
		    }
		    // Add in the date
		    translation.setAttribute("Created", DateFormat.getDateInstance().format(trans.getDate()));
		    // Add a text node with the actual text
		    translation.appendChild(doc.createTextNode(trans.getNote()));		    
		    // Add in the translation to the bubble
		    bubble.appendChild(translation);
		}
		// Now bubble has all the translations in it.  
		// Add the bubble to the doc fragment
		// System.out.println("Appending bubble "+bubble);
		frag.appendChild(bubble);
	    }
	    // Re-scale annotations
	    annPanel.scaleAnnotations(tempScale);
	    // Return the fragment
	    infoDisplay.setText("Created DOM for changes!");
	    // Now, get rid of the changes in the Vector - they called this, so we 
	    // assume that they updated the remote server.  This is because if we 
	    // we get called again, we only want to add the changes that have 
	    // happened since the last update...  
	    annPanel.clearChanges();
	    return frag;
	} catch (Exception e) {
	    System.out.println("Error trying to create a document: "+e);
	    e.printStackTrace();
	}
	// Re-scale annotations
	annPanel.scaleAnnotations(tempScale);
	// Only get here in an error case
	System.out.println("Wasn't able to create the document fragment for the page! Returning null...");
	return null;
    }


    /**
     * loadImage(Properties props, String imgName)
     *
     *  This method tries to load the image specified in imgName.  Note
     *  that imgName is only the name of the image itself with NO path
     *  information.  The property is checked for "imageSource" to 
     *  determine if we try to load over http or local file system.
     *  Similarly, we check out "imageHttpLocation" and "imageFileLocation"
     *  for the URL base and path respectively.
     * 
     *  I used to use an ImageIcon to load the images, but that didn't
     *  give me enough control over getting information on the loading 
     *  status.  So I have switched to making AnnotatePanel an ImageObserver
     *  ...  We will keep the ImageIcons
     *  that have loaded around in imageCache - a hashtable on the image
     *  names.  The ImageIcon is placed in the cache only after the image
     *  has completely loaded.  
     *
     *  If we are using the local file system, I just use an ImageIcon 
     *  because they are fast and easy.  We all have hard drives with like,
     *  10 ms seek times, right?  :) 
     *
     *  Since we now also can show images at different scales, we
     *  have to do something about that here.  In this case, we will
     *  default all images to display at 100 percent the first time
     *  they are put up on screen.  A little annoying for the user, 
     *  I know.  But much much easier on us.  (Because then we can 
     *  make some assumptions about the loaded status of images or not.)
     */
    public void loadImage(Properties props, String imgName) {
	infoDisplay.setText("Loading image "+imgName);
	
	// Set the image variables scale to 100%.
	currentFileName = imgName;
	currentImageHeight = -1;
	currentImageWidth = -1;
	currentImageScale = 100;
	// System.out.println("Reset the image width, height, scale to -1,-1,100");
	// Remeber to set the JComboBox to 100% also
	zoomLevel.setSelectedItem(zoom[3]);

	//  Check to see if we have the image in our imageCache
	if (imageCache.get(imgName) != null) {
	    // The image is in the cache. 
	    infoDisplay.setText("Found image "+imgName+" in cache.");
	    theBackground = (ImageIcon) imageCache.get(imgName);
	    // Set up the height, width
	    currentImageHeight = theBackground.getImage().getHeight(this);
	    currentImageWidth = theBackground.getImage().getWidth(this);
	    return;
	}
	if (props.getProperty("imageSource", "http").equals("http")) {
	    // Try to get images over a http connection.
	    String fileURL = new String(props.getProperty("imageHttpLocation", "http://fugu.cs.columbia.edu/images/Gto_01/") + imgName);
	    try {
		// Load the image.  Doesn't really load the pixels yet.
		Image newImage = Toolkit.getDefaultToolkit().getImage(new URL(fileURL)); 
		// set theBackground to null for a while so that the 
		// screen will just be blue or whatever while we load.
		theBackground = null;
		// Now we have to set ourselves up to monitor the image
		// through the imageUpdate() interface.
		// If we call getHeight or getWidth that will happen
		newImage.getHeight(this);
		/* We were going to use a MediaTracker to wait on the 
		   image here, but if we do that, we shut down the GUI.
		   So instead, we just set the background to null,
		   throw up the blue screen, and wait until the image
		   has finished loading.  When it has, we will catch
		   it in the imageUpdate() method, where we can set
		   up theBackground again nicely.
		   
		   There is a big problem with this though.  Besides us
		   totally not using ImageTacker which was supposed to
		   save us trouble and time.  That is that if the user
		   is impatient and clicks on another page to load, we
		   then have two images (generalize to n) loading, and
		   calling the imageUpdate() method.  We have to somehow
		   deal with that.  It is not clear to me that it will
		   be easy to deal with it.  For right now, we will assume
		   that we can only load one image at once.  Otherwise,
		   things get really nasty.
		   
		   Right now we have two varialbes, currentImageHeight
		   and currentFileName that track the height and name
		   of the current image loading.  If those change while
		   we are loading, our cache will be fucked.  At least
		   until the real image named whatever it is finishes
		   loading.  So maybe it isn't so bad.  But it certainly
		   is not good.  
		*/
	    } catch (Exception e) {
		System.out.println("Erorr loading http image ("+fileURL+")"+e);
	    }	    
	} else if (props.getProperty("imageSource").equals("file")) {
	    // Try to get images from the local file system.
	    /*  Big note here.  We are using the local file system, so 
	        we are not going to mess with any of that stuff that we
		did for loading the image.  That means we have to set the
		image height and width here as well, since the scaling 
		relies on those numbers to be set!!
	    */
	    String fileURL = new String(props.getProperty("imageFileLocation")+imgName);
	    try {
		theBackground = new ImageIcon(fileURL);
		// Set up the height, width
		currentImageHeight = theBackground.getImage().getHeight(this);
		currentImageWidth = theBackground.getImage().getWidth(this);
		infoDisplay.setText("Loaded file: "+fileURL);
		// Please put the image into the image cache
		imageCache.put(currentFileName, theBackground);
	    } catch (Exception e) {
		System.out.println("Erorr loading file image ("+fileURL+")"+e);
	    }
	} else {
	    System.out.println("I don't know how to get files using method: "+
			       props.getProperty("imageSource"));
	    infoDisplay.setText("I don't know how to get files using method: "+   props.getProperty("imageSource"));
	}
    }

    /**
     *  imageUpdate()
     * 
     *  Implemented so that we can be an ImageObserver.  We want to be
     *  able to tell how much of an image has loaded, so we can put up
     *  some sort of indication to the user that we are loading images.
     *  This should let us do that.  Maybe.
     */
    public boolean imageUpdate(Image img, int infoflags, int x, int y, int width, int height) {
	if ((infoflags & ImageObserver.ALLBITS) == ImageObserver.ALLBITS) {
	    // All bits have loaded
	    infoDisplay.setText("Finished loading "+currentFileName);
	} 
	if ((infoflags & ImageObserver.ABORT) == ImageObserver.ABORT) {
	    infoDisplay.setText("Aborted image loading for some reason.");
	} 
	if ((infoflags & ImageObserver.ERROR) == ImageObserver.ERROR) {
	    infoDisplay.setText("Image loading errored for some reason.");
	} 
	if ((infoflags & ImageObserver.FRAMEBITS) == ImageObserver.FRAMEBITS) {
	    infoDisplay.setText("We finished loading one frame of the image.");
	} 
	if ((infoflags & ImageObserver.WIDTH) == ImageObserver.WIDTH) {
	    infoDisplay.setText("We have access to the width: "+width);
	    currentImageWidth = width;
	    // System.out.println("Set the image width: "+width);
	} 
	if ((infoflags & ImageObserver.HEIGHT) == ImageObserver.HEIGHT) {
	    infoDisplay.setText("We have access to the height: "+height);
	    currentImageHeight = height; // Use this to calculate a % loading
	    // System.out.println("Set the image height: "+height);
	} 
	if ((infoflags & ImageObserver.SOMEBITS) == ImageObserver.SOMEBITS) {
	    infoDisplay.setText("Loaded "+ (100 * y)/currentImageHeight+
				"% of "+currentFileName);
	}
	if ((infoflags & ImageObserver.ALLBITS) == ImageObserver.ALLBITS) {
	    // We don't need to be notified about this image further.
	    // But we do need to set up the background with the fresh image!
	    /**
	     * Before setting the image, decouple the image from the 
	     * one that actually loaded.  It looks like it will re-load
	     * if you don't do this.  Thanks go to
	     * Andy Turner <andy@palomar.freeserve.co.uk>
	     * on comp.lang.java.programmer 
	     *
	     *  Sadly, this does not work.  It does not re-size the image, 
	     *  although we get a "first frame of image loaded" message.?
	     */
	    /* 
	       Image tmp = createImage (img.getWidth(null),
	       img.getHeight(null));
	       tmp.getGraphics().drawImage (img, 0, 0, null);
	       img = tmp;
	    */
	    theBackground = new ImageIcon(img);
	    // We also have to add the ImageIcon into the cache
	    // If we are at 100% scale, then put it in.  Don't otherwise.
	    if (currentImageScale == 100) {
		imageCache.put(currentFileName, theBackground);
	    } 
	    // And finally, have the background displayed
	    annPanel.setBackgroundImageOnly(theBackground);
	    return false;
	} else {
	    return true;
	}
    }

    public void buildUI(Container container, ImageIcon background,
			Properties props) {
	setBackground(Color.white);
	toolBar = buildToolBar(props);
	container.setLayout(new BorderLayout());
	container.add(toolBar, BorderLayout.NORTH);
        annPanel = new AnnotatorPanel(props, background);
	scrollPane = new JScrollPane(annPanel);
        container.add(scrollPane, BorderLayout.CENTER);
	//  Add a little text area down south to tell people 
	//  what is going on at times.
	infoDisplay = new JLabel("Setting up GUI...");
	container.add(infoDisplay, BorderLayout.SOUTH);
    }

    /**
     *  Set the AnnotationInformationPanel that we will use - pass
     *  it down to the annotate panel...
     */
    public void setAnnInfoPanel(AnnotationInformationPanel naip) {
	annInfoPanel = naip;
	annPanel.setAnnInfoPanel(naip);
	// Tell the annInfoFrame who will listen if notes change in some way.
	naip.setNoteChangeListener(annPanel);
    }

    /**
     *  Build the toolBar for the AnnotatePanel
     *  This will include three radio buttons for drawing 
     *  rectangles, ovals, and general paths.
     */
    public JToolBar buildToolBar(Properties props) {
	JToolBar bar = new JToolBar(JToolBar.HORIZONTAL);

	//  Create the radio buttons
	//  Draw general paths by default
	/*  If we are an applet, we should load the image icons from 
	    a URL based off of our code base.
	*/
	// Nazi java compiler can't recognize the valid initialization below
	JButton generalPath=null;
	JButton rectangle=null;
	JButton oval=null;
	JButton erase=null;
	if (props.getProperty("applet") != null && 
	    props.getProperty("applet").equals("true")) {
	    // Load buttons with URLs
	    try {
		generalPath = new JButton("polygon", new ImageIcon(new URL(props.getProperty("codebase")+"polygonDefault.jpg")));
		generalPath.setPressedIcon(new ImageIcon(new URL(props.getProperty("codebase")+"polygonSelected.jpg")));
		rectangle = new JButton("rectangle", new ImageIcon(new URL(props.getProperty("codebase")+"rectangleDefault.jpg")));
		rectangle.setPressedIcon(new ImageIcon(new URL(props.getProperty("codebase")+"rectangleSelected.jpg")));
		oval = new JButton("oval", new ImageIcon(new URL(props.getProperty("codebase")+"ellipseDefault.jpg")));
		oval.setPressedIcon(new ImageIcon(new URL(props.getProperty("codebase")+"ellipseSelected.jpg")));
		erase = new JButton("erase", new ImageIcon(new URL(props.getProperty("codebase")+"erase.gif")));
	    } catch (java.net.MalformedURLException ex) {
		System.out.println("Error trying to load icons for JToolBar images: Invalid URLs used: "+ props.getProperty("codebase") + "IMGFILE.EXT :"+ex);
	    }
	} else {
	    generalPath = new JButton("polygon", new ImageIcon("polygonDefault.jpg", "Polygon button"));	
	    generalPath.setPressedIcon(new ImageIcon("polygonSelected.jpg"));
	    rectangle = new JButton("rectangle", new ImageIcon("rectangleDefault.jpg", "Rectangle Button"));
	    rectangle.setPressedIcon(new ImageIcon("rectangleSelected.jpg"));
	    oval = new JButton("oval", new ImageIcon("ellipseDefault.jpg", "Ellipse Button"));
	    oval.setPressedIcon(new ImageIcon("ellipseSelected.jpg"));
	    erase = new JButton("erase", new ImageIcon("erase.gif", "Erase Bubble Button"));
	}
	generalPath.setSelectedIcon(generalPath.getPressedIcon());
	generalPath.setSelected(true);
	generalPath.addActionListener(this);
	rectangle.setSelectedIcon(rectangle.getPressedIcon());
	rectangle.addActionListener(this);
	oval.setSelectedIcon(oval.getPressedIcon());
	oval.addActionListener(this);
	//  Add the buttons to the tool bar
	bar.add(generalPath);
	bar.add(rectangle);
	bar.add(oval);
	//  Add a button for erasing things
	erase.addActionListener(this);
	bar.add(erase);
	//  Add zoomLevel comboBox
	bar.add(zoomLevel);

	return bar;
    }

    public void actionPerformed(ActionEvent e) {
	String command = e.getActionCommand();
	// If this is one of the drawing related buttons...
	if (command.equals("polygon") || command.equals("oval") ||
	    command.equals("rectangle") || command.equals("erase")) {
	    setToolbarButton(command);
	} else if (command.equals("comboBoxChanged")) {
	    // This is a command from the JComboBox.  Zoom!
	    /* When we request a scaled version of an image, it has
	       to load it again.  It might take a while.  So we do the
	       same thing that we would when we load an image normally.
	       We request the scaled image, then check the height
	       and register ourself as the image observer.  When 
	       the image is ready it will throw it up.  Also, this
	       will put the little % loaded message down at the
	       bottom, which is nice.
	       
	       The real work is done in the scaleImage() method though.
	       Here we just set a variable that tells us what percentage
	       to get the image at, and the request a loadImage().  
	    */

	    /*  Get the zoom level from the JComboBox iteself */
	    try {
		String selection = (String)zoomLevel.getSelectedItem();
		if (selection.equals("Fit Window")) {
		    // We should find out how to scale the image.
		    // Is the width, or the height the constraint?
		    System.out.println("Scaling to fit window...");
		    int pWidth = scrollPane.getViewport().getExtentSize().width;
		    int pHeight = scrollPane.getViewport().getExtentSize().height;
		    int imgWidth = theBackground.getIconWidth();
		    int imgHeight = theBackground.getIconHeight();
		    int wScale = (100*pWidth) / imgWidth;
		    int hScale = (100*pHeight) / imgHeight;
		    System.out.println("wScale: "+wScale+" hScale: "+hScale);
		    //  Choose the lower of the two scales
		    currentImageScale = wScale < hScale ? wScale : hScale;
		    /* Don't set the zoomBox to that percentage because
		       then we would have to mess around with adding that
		       one into zoomLevel, and removing it later...
		    */
		} else if (selection.length() > 0) {
		    // It is probably a %.  Remove the last %.
		    String num = "";
		    if (selection.charAt(selection.length()-1) == '%') {
			num = selection.substring(0, selection.length()-1);
		    } else {
			num = selection;
		    }
		    currentImageScale = Integer.parseInt(num);
		}
	    } catch (NumberFormatException ex) {
		System.out.println("The combo box isn't a number: " +
				   zoomLevel.getSelectedItem() + ": " + ex);
		// Default back to making a full size image
		currentImageScale = 100;
	    }
	    // currentImageScale tells us the % we should be at.
	    scaleImage();
	}
    }

    /**
     *  void scaleImage() 
     *   
     *  We want a scaled version of the current background image. 
     *  First, we have to make sure that the image is loaded.  
     *  It will not be loaded if either currentImageHeight == -1
     *  or currentImageWidth == -1;  So check on those.  
     *
     *  For right now, we will just loop with a bit of sleeping.
     *  To do this *RIGHT* we will want to kick up a new thread 
     *  to do that checking for us.  
     *
     *  Once we know that the image is loaded, we will get a scaled
     *  version of the image, giving it ourself as the imageobserver.
     *  Then that will throw the image up.  yay.
     *
     *  Also, we have to re-scale all of the Annotations in the 
     *  AnnotatorPanel.  We'll pass a message on to it, and then 
     *  let it take care of re-scaling all of the Annotations.
     */

    void scaleImage() {
	// Make sure we have and image to work with ...
	if (theBackground == null || currentFileName.equals("")) {
	    return;
	}
	
	// System.out.println("Scaling "+ currentFileName +" to "+currentImageScale);
	while (currentImageHeight == -1 || currentImageWidth == -1) {
	    // System.out.println("Going to sleep Height: "+currentImageHeight+" Width: "+currentImageWidth);
	    try {
		infoDisplay.setText("Sleeping while image is loading...");
		Thread.sleep(1000);
	    } catch (Exception e) {
		// System.out.println("I'm tired and want to sleep while I am waiting for the image to load to scale it, but: "+e);
	    }
	}
	// System.out.println("Left the while loop, going to work.");
	/* 
	   Image is loaded.  Get the new width, height using scale
	   Also note.  We can't use the values in currentImageWidth
	   or currentImageHeight.  Why not?  Well, those are the values
	   of the current image.  If someone displayed the image at 25%,
	   then that is what they are.  When they scale back up again,
	   well...  So we have to first get the real image.  Luckily,
	   it will be in our imageCache.  Yay.
	*/
	int newWidth, newHeight;
	Image realImage = ((ImageIcon)imageCache.get(currentFileName)).getImage();
	if (realImage != null) {
	    infoDisplay.setText("Scaling image to "+currentImageScale);
	    currentImageWidth = realImage.getWidth(this);
	    currentImageHeight = realImage.getHeight(this);
	    newWidth = currentImageWidth * currentImageScale / 100;
	    newHeight = currentImageHeight * currentImageScale / 100;
	    // Now request the re-scaled image
	    Image scaledImage = realImage.getScaledInstance(newWidth, newHeight, Image.SCALE_DEFAULT);
	    // Now make us the imageobserver for this thing.
	    // Remember, the image will be displayed in imageUpdate();
	    scaledImage.getHeight(this);
	    // Now scale the annotations
	    annPanel.scaleAnnotations(currentImageScale);
	} else {
	    infoDisplay.setText("Error retrieving real image to scale!!");
	    System.out.println("There was no real image in the iamgeCache to use to scale from!!");
	}
    }

    /**
     *  The String argument button should be one of "polygon" 
     *  "rectangle" or "oval".  This function will deselect all
     *  buttons other than the one specified in button, and set
     *  the AnnotatorPanel to drawing objects of the type specified
     *  in button.
     */
    public void setToolbarButton(String button) {
	if (button.equals("polygon")) {
	    ((JButton)toolBar.getComponentAtIndex(0)).setSelected(true);
	    // Draw polygons
	    annPanel.setCurrentShapeType("polygon");
	} else {
	    ((JButton)toolBar.getComponentAtIndex(0)).setSelected(false);
	}
	if (button.equals("rectangle")) {
	    ((JButton)toolBar.getComponentAtIndex(1)).setSelected(true);
	    // Draw rects
	    annPanel.setCurrentShapeType("rectangle");
	} else {
	    ((JButton)toolBar.getComponentAtIndex(1)).setSelected(false);
	}
	if (button.equals("oval")) {
	    ((JButton)toolBar.getComponentAtIndex(2)).setSelected(true);
	    // Draw ovals
	    annPanel.setCurrentShapeType("oval");
	} else {
	    ((JButton)toolBar.getComponentAtIndex(2)).setSelected(false);
	}	
	if (button.equals("erase")) {
	    ((JButton)toolBar.getComponentAtIndex(3)).setSelected(true);
	    // Erase bubbles!
	    annPanel.setCurrentShapeType("erase");
	} else {
	    ((JButton)toolBar.getComponentAtIndex(3)).setSelected(false);
	}
    }

    public static void main(String s[]) {
	System.out.println("This isn't a stand alone class.");
    }
    
    /**
     *  Return a vector of all the changes that we have made since
     *  the page was loaded.  Vector will not be null, but it could
     *  be empty
     */
    public Vector getChanges() {
	return annPanel.getChanges();
    }

    public void setAnnotator(Annotator a) {
	annPanel.setAnnotator(a);
	currentAnnotator = a;
    }

} // AnnotatePanel

/**
 *  The AnnotatorPanel takes care of actually drawing the 
 *  annotation GeneralShape objects that are inside of it. 
 *  
 *  It implements MouseListener so that it can capture clicks to the
 *  panel to create the GeneralShape.  It is only interested in clicks
 *  and double clicks.  When there is a click inside the panel, we
 *  will add a new point to the current shape object we are defining.
 *  A double click will close the shape object, and any subsequent
 *  clicks will create a new current shape.  
 *
 *  An AnnotatorPanel requires a Properties object when created.  The
 *  Properties object sets some values for the colors used in the
 *  panel.  They relevant keys are: Annotator.backgroundColor (sets
 *  the color of the background of the panel)
 *  Annotator.Annotation.fgcolor (sets the line color for the
 *  annotation) and Annotator.Annotation.gbcolor (set the bg color of
 *  the annotation, or none for none.)  The current implementation of
 *  the properties checking is inefficient - the code for checking
 *  colors should be done once, not every time through the paint
 *  function.  The values passed are just strings that represent
 *  integers.  If they are less than 0 or greater than whatever an int
 *  is in Java, badness should ensue.  I should check this and put in
 *  some exception handling.
 *
 *  Now implementing "Scrollable" interface for your scrolling pleasure!
 *
 *  Clicking is a little bit sensitive.  (Don't go to fast or too slow.)
 */
class AnnotatorPanel extends JPanel implements MouseListener, MouseMotionListener, Scrollable {
    Vector annotations;
    Vector changes;  // Vector of NEW annotations since we loaded the page.
    Shape currentShape;
    Rectangle area;
    Properties opts;  // options for the panel
    ImageIcon background;  // The background we should have.  null for none.
    boolean firstTime; // First time through paint loop or not. 
    Image bgbuf;  // Store background in here.
    int mouseX, mouseY;  /** Location of mouse currently -- note that these
			     points are always going to be in the native (100)
			     scale -- so when we set them if we are scaled, 
			     we put that back to what they would be if we 
			     were not scaled.  Got that?? */
    Annotator currentAnnotator;  // Current Annotator
    String currentShapeType = "polygon";  // What type are we drawing now?
    AnnotationInformationPanel annInfoPanel;  // Where to send annotation information stuff to? 
    int maxUnitIncrement = 5;  //  How much to scroll per increment.  
    int currentScale = 100;  // Always 100 when we start 

    public AnnotatorPanel(Properties props, ImageIcon bkgd){
	opts = props;  
	area = new Rectangle();
	annotations = new Vector();	
	changes = new Vector();
	addMouseListener(this);
	addMouseMotionListener(this);
	firstTime = true;
	background = bkgd;
	
	//  Add a component listener to notice when we resize
	this.addComponentListener(new ComponentAdapter() {
	    public void componentResized(ComponentEvent e) {
		// Do resize stuff here.  Can't change the 
		// background now though: because of they way it is
		// set up, we can't tell if we have a real image as
		// a background or just a dummy full color image. :(
	    }
	});

	if (bkgd != null) {
	    //  Make our preferred size the size of the bkgd image	
	    setPreferredSize(new Dimension(bkgd.getIconWidth(), bkgd.getIconHeight()));
	} else {
	    setPreferredSize(new Dimension(600, 600));
	}
	
	// We also have to make sure we resize ourself!
	revalidate();
    }

    /**
     *  scaleAnnotations(int scale) 
     *  Tells use that we are supposed to scale all of the annotations
     *  to a new scale.  We will keep this value around so that we 
     *  know what scale we are drawing at.
     *
     *  We also have to remember that we have to take care to look at
     *  scale when we are paintComponent 'ing because we will have to 
     *  draw the incompleted Annotations to scale as well.  
     */
    public void scaleAnnotations(int scale) {
	// Remember our current scale.
	currentScale = scale;
	// Scale each annotation
	for (int i=0; i<annotations.size(); i++) {
	    Annotation note = (Annotation) annotations.get(i);	    
	    note.scale(scale);
	}
    }

    /**
     *  setBackground
     *  lets us change the background.  This should be called when
     *  we change the Page, and want to throw up the new images and
     *  the new annotations.  
     *
     *  If you just want to throw up a new image, called 
     *  setBackgroundImageOnly(ImageIcon back)
     */
    public void setBackground(ImageIcon back) {
	background = back;
	bgbuf = null;
	// Force us to re-create the image buffer 
	firstTime = true;
	// Also, clear out all Annotations.
	// If someone is giving us a new page, they should 
	// also insert some new annotations pretty soon.  
	annotations = new Vector();
	// Clear out changes that have been made
	changes = new Vector();

	if (back != null) {
	    setPreferredSize(new Dimension(back.getIconWidth(), back.getIconHeight()));
	}
	// We will also re-set the scale 
	currentScale = 100; 
	
	// Try to get us to re-size ourselves
	invalidate();
	revalidate();
	repaint();
    }

    /**
     *  setBackgroundImageOnly(ImageIcon back)
     *
     *  Sets the background image only.  Does not clear out the 
     *  Annotations that already exist.  Only called really from
     *  imageUpdate() when we are done loading an image.  
     */
    public void setBackgroundImageOnly(ImageIcon back) {
	background = back;
	bgbuf = null;
	// Force us to re-create the image buffer 
	firstTime = true;
	if (back != null) {
	    setPreferredSize(new Dimension(back.getIconWidth(), back.getIconHeight()));
	}
	invalidate();
	revalidate();
	repaint();
    }

    public void setProperties(Properties props) {
	opts = props;
    }

    /* Functions added to implement scrollable interface */
    public Dimension getPreferredScrollableViewportSize() {
	return getPreferredSize();
    }

    public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation, int direction) {
	
        int currentPosition = 0;
        if (orientation == SwingConstants.HORIZONTAL)
            currentPosition = visibleRect.x;
        else
            currentPosition = visibleRect.y;
	
        if (direction < 0) {
            int newPosition = currentPosition - (currentPosition / maxUnitIncrement) * maxUnitIncrement;
            return (newPosition == 0) ? maxUnitIncrement : newPosition;
        } else {
            return ((currentPosition / maxUnitIncrement) + 1) * maxUnitIncrement - currentPosition;
        }
    }

    public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation, int direction) {
        if (orientation == SwingConstants.HORIZONTAL)
            return visibleRect.width - maxUnitIncrement;
        else
            return visibleRect.height - maxUnitIncrement;
    }

    public boolean getScrollableTracksViewportWidth() {
        return false;
    }
    
    public boolean getScrollableTracksViewportHeight() {
        return false;
    }

    public void setMaxUnitIncrement(int pixels) {
        maxUnitIncrement = pixels;
    }

    /*  Done with Scrollable functions */

    public void setAnnInfoPanel(AnnotationInformationPanel aip) {
	annInfoPanel = aip;
    }
    
    public void setCurrentShapeType(String type) {
	currentShapeType = type;
    }
    
    public Vector getAnnotations() {
	return annotations;
    }

    public void setAnnotator(Annotator a) {
	currentAnnotator = a;
	annInfoPanel.setAnnotator(a);
    }

    public Annotator getAnnotator() {
	return currentAnnotator;
    }

    public Vector getChanges() {
	return changes;
    }

    /**
     *  annotationUpdated(Annotation a, int noteNumber)
     *  tells us that the note number noteNumber has changed.  This
     *  can be caused by two things: someone created a new note in
     *  an existing annotation, or someone edited a note in an existing
     *  annotation.  If Annotation a is not in changes() then we should
     *  put it there.  
     */
    public void annotationUpdated(Annotation a, int noteNumber) {
	if (!changes.contains(a)) {
	    // We do not have this annotation in changes()
	    changes.add(a);
	}
    }

    /**
     *  clearChanges()
     *  Clear out the changes
     */
    public void clearChanges() {
	changes = new Vector();
    }

    public void paintComponent(Graphics g){
	// super.paintComponent(g);
	Graphics2D g2 = (Graphics2D)g;

	//  Do some stuff if it is the first time through
	if (firstTime) {
	    if (background == null) {
		//  Create an image that is the background color
		Dimension dim = getSize();
		int w = dim.width;
		int h = dim.height;
		int x = w/2 + 50;
		int y = h/2 - 25;
		Rectangle area = new Rectangle(w, h);
		bgbuf = createImage(w, h);
		Graphics2D big = (Graphics2D) bgbuf.getGraphics();
		// Clears background to color in properties
		String bgcolor = 
		    opts.getProperty("Annotator.backgroundColor", "32767");
		big.setColor(new Color(Integer.parseInt(bgcolor)));
		// g2.setColor(Color.white);
		big.fillRect(0, 0, w, h);
		// Set background to the image
		background = new ImageIcon(bgbuf, "Background");
	    }
	    // Get the image from the ImageIcon, stuff into big
	    bgbuf = background.getImage();
	    
	    firstTime = false;
	}

	// Draw the background	
	g2.drawImage(bgbuf, 0,0, this);
	
	// Set color for drawing out single shape
	String shapefgcolor =
	    opts.getProperty("Annotator.Annotation.fgcolor", "16000");
	g2.setColor(new Color(Integer.parseInt(shapefgcolor)));
	// g2.setColor(Color.magenta);
	g2.setStroke(new BasicStroke(3.0f));
	String shapefill =
	    opts.getProperty("Annotator.Annotation.bgcolor", "none");

	/*  Draws the current shape - if it is a General Path this 
	    draws all the points on the path up to the last one
	    Otherwise, we are keeping the oval and rect shapes to be
	    width/height of 0 so this won't draw anything.  
	    
	    Also, remember to keep track of currentScale
	*/
	if (currentShape != null) {
	    // Fill the shape first if we are supposed to
	    if (! shapefill.equals("none") ) {
		g2.setColor(new Color(Integer.parseInt(shapefill)));
		if (currentScale == 100) {
		    g2.fill(currentShape);
		} else {
		    /* We have to fill a scaled instance of this shape
		       The good news is we only have to do this for a
		       GeneralPath.  Right?  That might not be true...
		       I don't know, I keep shapefill to none usually...
		    */
		    if (currentShape instanceof GeneralPath) {
			g2.fill(((GeneralPath)currentShape).createTransformedShape(AffineTransform.getScaleInstance(((double)currentScale) / ((double)100), ((double)currentScale) / ((double)100))));
		    } 
		}
		g2.setColor(new Color(Integer.parseInt(shapefgcolor)));
	    }
	    // Same thing here with scale 
	    if (currentScale == 100) {
		g2.draw(currentShape);
	    } else {
		/* We have to draw a scaled instance of this shape
		   The good news is we only have to do this for a
		   GeneralPath.  Right?  Because they are the only 
		   things that have a "completed" portion and then an 
		   "intermediate" portion
		*/
		if (currentShape instanceof GeneralPath) {
		    g2.draw(((GeneralPath)currentShape).createTransformedShape(AffineTransform.getScaleInstance(((double)currentScale) / ((double)100), ((double)currentScale) / ((double)100))));
		} 
	    }
	}
	
	// Draw the intermediate portion of the current shape 
	// ie the current shape + what it woud look like if it was closed now.
	if (currentShape != null) {
	    // Draw a line to the current mouse location if a polygon
	    if (currentShapeType.equals("polygon")
		&& ((GeneralPath)currentShape).getCurrentPoint() != null ) { 
		int lastX = (int) ((GeneralPath)currentShape).getCurrentPoint().getX();
		int lastY = (int) ((GeneralPath)currentShape).getCurrentPoint().getY();
		// Check to see if we are scaled
		if (currentScale == 100) {
		    //  Thank god, we aren't.
		    g2.drawLine(mouseX, mouseY, lastX, lastY);
		} else {
		    // Damn, we are scaled.  
		    lastX = (lastX * currentScale) / 100;
		    lastY = (lastY * currentScale) / 100;
		    mouseY = (mouseY * currentScale) / 100;
		    g2.drawLine(mouseX, mouseY, lastX, lastY);
		}
	    } else if (currentShapeType.equals("oval") ||
		       currentShapeType.equals("rectangle")) {
		// Get the X and Y coord first clicked at.
		// based on mouseX and mouseY find true upper-left corner
		int startX = (int) ((RectangularShape)currentShape).getX();
		int startY = (int) ((RectangularShape)currentShape).getY();
		int ULX, ULY;
		int height, width;
		if (startX > mouseX) 
		    ULX = mouseX;
		else 
		    ULX = startX;
		
		if (startY > mouseY) 
		    ULY = mouseY;
		else 
		    ULY = startY;
		height = Math.abs(mouseY - startY);
		width = Math.abs(mouseX - startX);
		/*  We have to check to see if we are scaled because
		    startX/Y comes from the currentShape - it is in native
		    (scale 100) coordinates) and mouseX and mouseY are
		    also in native coordinates.
		*/
		if (currentScale != 100) {
		    ULX = (ULX * currentScale) / 100;
		    ULY = (ULY * currentScale) / 100;
		    width = (width * currentScale) / 100;
		    height = (height * currentScale) / 100;
		}
		//  Now draw the rect or oval
		if (currentShapeType.equals("rectangle")) {
		    g2.drawRect(ULX, ULY, width, height);
		} else {
		    g2.drawOval(ULX, ULY, width, height);
		}
	    }     
	}
	
	// Draws the completed annotations
	// But not if we don't want to draw the shapes
	if (opts.getProperty("drawAnnotationOutlines", "true").equals("true")) {
	    // See if we should fill or not
	    boolean fill;
	    Color fillColor = Color.white;
	    if (opts.getProperty("Annotator.Annotation.bgcolor") != null &&
		opts.getProperty("Annotator.Annotation.bgcolor").equals("none")) {
		fill = false;
	    } else {
		fill = true;
		fillColor = new Color(Integer.parseInt(opts.getProperty("Annotator.Annotation.bgcolor", "16000")));
	    }	    
	    // Get the settings for the language we want to display
	    // Default to English
	    String lang = opts.getProperty("display.AnnotationLanguage", "English");
	    // Get the settings for the annotator we want to display
	    // Default to nothing
	    String annID = opts.getProperty("display.Annotator", "");
	    for (int i=0; i<annotations.size(); i++) {
		Annotation note = (Annotation) annotations.get(i);	    
		if (fill) {
		    Note theNote = note.getNote(annID, lang);
		    String textToShow = theNote == null ? "" : theNote.getNote();
		    note.drawSelf(g2, fill, fillColor, textToShow);
		} else {
		    note.drawSelf(g2);
		}
	    }
	}
    }

    public void addAnnotation(Annotation a) {
	annotations.add(a);
    }
    
    public void mouseClicked (MouseEvent e) {
	/*  Check to see if we have a current annotator
	    If not, throw up a dialog telling them to log in
	*/
	if (currentAnnotator == null) {
	    JOptionPane.showMessageDialog(this, "To annotate the image, you must first log in as an Annotator.\nPlease log in using the \"Log in\" option from the \"Annotator\" menu.", "Please log in", JOptionPane.INFORMATION_MESSAGE);
	    return;
	}
	/*  Check cases for polygon, rectangle, and oval
	 */	
	if (currentShapeType.equals("polygon")) {
	    /* If we have only one click, add a new point to the
	       current shape object that we are defining. 
	    */
	    float x1, y1;
	    x1 = (float) e.getX();
	    y1 = (float) e.getY();
	    //  If we are scaled, scale the points to where we should really be
	    if (currentScale != 100) {
		x1 = (x1 * 100) / currentScale;
		y1 = (y1 * 100) / currentScale;
	    }
	    if (e.getClickCount() == 1) {
		//  We are defining a new point to add to the shape
		//  If the shape is null, create a new one
		if (currentShape == null) {
		    currentShape = new GeneralPath();
		    //  Add the first point as a point only
		    ((GeneralPath)currentShape).moveTo(x1, y1);
		} else {
		    //  Add the point to the currentShape
		    ((GeneralPath)currentShape).lineTo(x1, y1);
		}
	    } else if (e.getClickCount() >= 2 && currentShape != null) {
		//  Close off the current shape.
		((GeneralPath)currentShape).closePath();
		//  Add this shape to the vector of annotations
		Annotation tempAnnotation = new Annotation(currentShape);
		String shapefill =
		    opts.getProperty("Annotator.Annotation.bgcolor", "none");
		if (! shapefill.equals("none") ) {
		    tempAnnotation.setFillColor(new Color(Integer.parseInt(shapefill)));
		    tempAnnotation.setFill(true);
		}
		// If we are scaled, request scaled version of annotation
		if (currentScale != 100) {
		    tempAnnotation.scale(currentScale);
		}
		annotations.add(tempAnnotation);
		//  This is a change that we made.  Remember it.
		changes.add(tempAnnotation);
		//  Reset this shape to null
		currentShape = null;
	    }		
	} else if (currentShapeType.equals("rectangle") || 
		   currentShapeType.equals("oval")) {
	    /* If we have no current shape and there is one click,
                start to define a new rectangle/oval current shape.
                If we have a current shape and there is one click,
                then we can close the current shape off and set
                currentShape to null again. */
	    if (currentShape == null) {
		//  The shape is null.  Start a new shape
		//  Put the coordinate that they clicked on as the
		//  upper-left coord.  Make the width / height to be 0
		double ptX = e.getX();
		double ptY = e.getY();
		// Check to see if we are scaled
		if (currentScale != 100) {
		    ptX = (ptX * 100) / currentScale;
		    ptY = (ptY * 100) / currentScale;
		}
		if (currentShapeType.equals("rectangle")) {
		    currentShape = new Rectangle2D.Double(ptX, ptY, 0, 0);
		} else {
		    currentShape = new Ellipse2D.Double(ptX, ptY, 0, 0);
		}
	    } else {
		//  We have a current shape.  Close it off.
		//  Add this shape to the vector of annotations
		//  We have to find the upper-left corner of the shape
		//  Get the X and Y coord first clicked at.
		//  based on mouseX and mouseY find true upper-left corner
		int startX = (int) ((RectangularShape)currentShape).getX();
		int startY = (int) ((RectangularShape)currentShape).getY();
		int ULX, ULY;
		int height, width;
		int ptX = e.getX();
		int ptY = e.getY();
		if (currentScale != 100) {
		    ptX = (ptX * 100) / currentScale;
		    ptY = (ptY * 100) / currentScale;
		}
		if (startX > ptX) 
		    ULX = ptX;
		else 
		    ULX = startX;
		
		if (startY > ptY) 
		    ULY = ptY;
		else 
		    ULY = startY;

		height = Math.abs(ptY - startY);
		width = Math.abs(ptX - startX);

		//  Create the shape		
		if (currentShapeType.equals("rectangle"))
		    currentShape = new Rectangle2D.Double(ULX, ULY, width, height);
		else 
		    currentShape = new Ellipse2D.Double(ULX, ULY, width, height);  
		Annotation tempAnnotation = new Annotation(currentShape);
		String shapefill =
		    opts.getProperty("Annotator.Annotation.bgcolor", "none");
		if (! shapefill.equals("none") ) {
		    tempAnnotation.setFillColor(new Color(Integer.parseInt(shapefill)));
		    tempAnnotation.setFill(true);
		}
		
		// If we are scaled, request the scaled version of Annotation
		if (currentScale != 100) {
		    tempAnnotation.scale(currentScale);
		}
		annotations.add(tempAnnotation);
		//  This is a change that we made.  Remember it.
		changes.add(tempAnnotation);
		//  Reset this shape to null
		currentShape = null;
	    }
	} else if (currentShapeType.equals("erase")) {
	    //  Erase the bubble we clicked on. 
	    int i=0;
	    while (i<annotations.size()) {
		//  If this annotation contains the point and 
		//  it is not the same annotation that is already set
		Annotation temp = (Annotation) annotations.elementAt(i);
		/* mouseX and mouseY are *NOT* scaled.  Must scale them.
		 */
		if (temp.getShape().contains((mouseX*currentScale)/100, (mouseY*currentScale)/100)) {
		    // Delete this Annotation
		    annotations.removeElementAt(i);
		    // Delete it from chanes too, if it exists
		    changes.remove(temp);
		    i = annotations.size();
		}
		i++;
	    }
	}
	//  Request a re-draw of the panel.
	this.repaint();
    }
    public void mousePressed(MouseEvent e) {
    }
    public void mouseReleased(MouseEvent e) {
    }
    public void mouseEntered(MouseEvent e) {
    }
    public void mouseExited(MouseEvent e) {
    }
    /**
     *  Support the ability to draw partially complete shapes
     *  by updating the current location of the mouse. 
     *  Also, if we are not drawing a new shape, see if we 
     *  are over a current shape that has been defined.  If so,
     *  select it and alert the AnnotationInformationPanel if 
     *  it is not null
     */  
    public void mouseMoved(MouseEvent e) {
	mouseX = e.getX();
	mouseY = e.getY();
	// Check for scale
	if (currentScale != 100) {
	    mouseX = (mouseX * 100) / currentScale;
	    mouseY = (mouseY * 100) / currentScale;
	}
	this.repaint();
	// Get the settings for the language we want to display
	// Default to English
	String lang = opts.getProperty("display.AnnotationLanguage", "English");
	// Get the settings for the annotator we want to display
	// Default to nothing
	String annID = opts.getProperty("display.Annotator", "");
	if (currentShape == null) {
	    //  Check to see if we are over a defined annotation	
	    //  Look at each Annotation and check to see if e is in it or not
	    int i=0;
	    while (i<annotations.size()) {
		//  If this annotation contains the point and 
		//  it is not the same annotation that is already set
		//  Have to scale mouseX and mouseY, since they scaled 
		//  to the native scale...
		Annotation temp = (Annotation) annotations.elementAt(i);
		if (temp.getShape().contains((mouseX*currentScale)/100, (mouseY*currentScale)/100) && annInfoPanel.getAnnotation() != temp) {
		    // Set this annotation as the current annotation 
		    // in the AnnotationInformationPanel
		    annInfoPanel.setAnnotation(temp);
		    i = annotations.size();
		    //  Also, put the first translation in the infoDisplay
		    if (temp.getNumNotes() > 0) {
			AnnotatePanel.infoDisplay.setText(temp.getNote(annID, lang).getCustomFieldValue("Character")+": "+temp.getNote(annID, lang).getNote());
		    }
		}
		i++;
	    }
	}
    }
    public void mouseDragged(MouseEvent e) {
    }
}



