
/**
 * GMAOApplet.java
 *
 *  I'm going to try to make GMAO run as an applet.
 *  Good luck.  
 *
 * Created: Thu Jul 13 16:38:51 2000
 *
 * @author David Evans
 * @version
 */

import javax.swing.*;
import java.util.Properties;
import java.awt.event.*;

/* 
   GMAO - Image Annotation for Manga Translation
   Copyright (C) 2000  David Kirk Evans
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

   David Kirk Evans (devans@cs.columbia.edu)
   http://www.cs.columbia.edu/~devans/   
*/


public class GMAOApplet extends JApplet {
    Properties props;
    String paramInfo[][] = {
	{"imageLocation",    "url",    "Location to load images from."},
	{"imageFileLocation", "path", "Location of image files on disk, if using the local file system."},
	{"imageSource",   "http or file",     "Load images from http source or local file system."},
	{"xmlFile", "url", "Location of the .xml file to parse for manga translation."},
	{"drawAnnotationOutlines", "true or false", "Should we draw the outlines of the annotations or not?"},
	{"display.AnnotationLanguage", "Language name", "Name of the preferred language of annotations to display"},
	{"display.Annotator", "AnnotatorID", "ID of the preferred Annotator to display"},
	{"Annotator.Annotation.bgcolor", "Number that can parse to a color", "Color for the background of annotations, none for transparent"},
	{"Annotator.Annotation.fgcolor", "Number that can parse to a color", "Color for the foreground drawing of annotations.  Text too."}
    };
    GMAOGUI gui;
    
    public GMAOApplet() {
    }
    
    public void init() {
	// First, we have to set up our properties file.
	props = new Properties();
	// Set up some default values
	props.put("drawAnnotationOutlines", "true");
	props.put("memoryLimit", "1024");
	props.put("Annotator.Annotation.bgcolor", "-1");
	props.put("Annotator.Annotation.fgcolor", "16000");
	props.put("Annotator.backgroundColor", "32767");
	props.put("compressXMLTreeView", "true");
	props.put("imageFileLocation", "c\\:/inetpub/wwwroot/images/Gto_02/");
	props.put("compressXMLTreeView", "true");
	props.put("display.AnnotationLanguage", "English");
	props.put("display.Annotator", "devans");
	// Applets should only access stuff over the web.
	// At a later date, allow user to change to file
	props.put("imageSource", "http");
	// Default to Fugu, GTO01
	props.put("imageHttpLocation", "http://fugu.cs.columbia.edu/images/Gto_01/");
	props.put("xmlFile", "http://fugu.cs.columbia.edu/images/Gto_01/GTO01.xml");
	// The name of the server to look for rmiregistry for /GMAOServer 
	props.put("updateServer", "fugu.cs.columbia.edu");
	props.put("cacheSize", "3");
	// Try to load in some PARAMETER settings.
	String parm = getParameter("imageHttpLocation");
	if (parm != null) {
	    props.put("imageHttpLocation", parm);
	}
	parm = getParameter("imageFileLocation");
	if (parm != null) {
	    props.put("imageFileLocation", parm);
	}
	parm = getParameter("imageSource");
	if (parm != null) {
	    props.put("imageSource", parm);
	}
	parm = getParameter("xmlFile");
	if (parm != null) {
	    props.put("xmlFile", parm);
	}
	parm = getParameter("display.AnnotationLanguage");
	if (parm != null) {
	    props.put("display.AnnotationLanguage", parm);
	}
	parm = getParameter("display.Annotator");
	if (parm != null) {
	    props.put("display.Annotator", parm);
	}
	parm = getParameter("drawAnnotationOutline");
	if (parm != null) {
	    props.put("drawAnnotationOutline", parm);
	}
	parm = getParameter("Annotator.Annotation.bgcolor");
	if (parm != null) {
	    props.put("Annotator.Annotation.bgcolor", parm);
	}
	parm = getParameter("Annotator.Annotation.fgcolor");
	if (parm != null) {
	    props.put("Annotator.Annotation.fgcolor", parm);
	}
	// Tell people that we are an applet
	props.put("applet", "true");
	
	// Set the codebase
	props.put("codebase", getCodeBase().toString());
	//  Create the GMAOGUI
	gui = new GMAOGUI("GMAO2k", props);
	gui.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {gui.hide();}
        });
	
        gui.pack();
        gui.setVisible(true);

	/* Check to see if we have a XML file specified.  If we do,
	   load the document straight away.
	*/
	if (props.getProperty("xmlFile") != null) {
	    AnnotatePanel.infoDisplay.setText("Auto-loading "+props.getProperty("xmlFile")+" please wait...");
	    try {
		gui.openXMLFile(props.getProperty("xmlFile"));
	    } catch (java.io.UnsupportedEncodingException ioe) {
		JOptionPane.showMessageDialog(gui, "Fatal Error: This Java Environment doesn't seem to support japanese file encoding methods!\nLook for instructions on how to download i18n.jar.\nAlso, applet is now rendered totally useless.", "Jikusyo! Fatal Error!", JOptionPane.ERROR_MESSAGE);
	    }
	    
	}
	
    }

    public String[][] getParameterInfo() {
	return paramInfo;
    }

    public void start() {

    }

    public void stop() {

    }

    public void destroy() {

    }

} // GMAOApplet
