
/**
 * GMAOCreditsBox.java
 *
 *  A simple little panel that will scroll up a text message
 *  when clicked on.  I actually am just re-using some code 
 *  I wrote for Carl and my TIC-TAC-TOE project.  
 * 
 *  Would like to add support for images as well.  
 *
 * Created: Fri Aug 04 17:30:46 2000
 *
 * @author David Evans
 * @version
 */

/* 
   GMAO - Image Annotation for Manga Translation
   Copyright (C) 2000  David Kirk Evans
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

   David Kirk Evans (devans@cs.columbia.edu)
   http://www.cs.columbia.edu/~devans/   
*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class GMAOCreditsBox extends JPanel implements Runnable {
    private Font textFont;
    private FontMetrics textFontMetrics;
    private boolean fontsSetup = false;
    private boolean scrolling = true;  // are we currently scrolling?
    private int scrolly = 0;  // Where have we scrolled to
    private int currentIndex = 0;  // Current string index
    private String[] title = {"Great Manga Applet Onidzuka", 
			      "Coding / Etc:",
			      "Dave Evans",
			      "Thanks to:",
			      "Onidzuka Eikichi",
			      "Adam M. Costello (Beta Tester #1)",
			      "Your name here - contact me!"
                             };
    public static final int CREDIT_BOX_HEIGHT = 100;
    public static final int CREDIT_BOX_WIDTH = 200;
    public static final int CREDIT_SCROLL_INCREMENT = 2;

    public GMAOCreditsBox() {
	setBackground(Color.white);
	addMouseListener( new MouseAdapter() {
	    public void mouseClicked(MouseEvent e) {
		//  They clicked the mouse.  Tell the panel to start
		//  scrolling
		toggleScroll();
	    }
	});
	Dimension mySize = getSize();
	setPreferredSize(new Dimension(CREDIT_BOX_WIDTH, CREDIT_BOX_HEIGHT));
	scrolly = CREDIT_BOX_HEIGHT / 2;
	new Thread( this ).start();
	validate();
	repaint();
}

    public GMAOCreditsBox(String t) {
	this();
	this.title[0] = t;
    }

    public void setTitle(String t) {
	this.title[0] = t;
    }

    private void setFonts (Graphics g) {
	if (fontsSetup) return;
	textFont = new Font("System", Font.PLAIN, 14);
	textFontMetrics = g.getFontMetrics(textFont);
	fontsSetup = true;
    }
    
    public void paintComponent(Graphics g) {
	// Set up the fonts if they have not been already
	setFonts(g);	
	// Find out where to draw the string so it is centered
	Dimension ourSize = getSize();
	int titleWidth = textFontMetrics.stringWidth(title[currentIndex]);
	int client_width = ourSize.width;
	int client_height = ourSize.height;
	int titleX = (client_width - titleWidth) / 2;
	int titleY = client_height - scrolly;
	// Clear the background 
	g.clearRect(0,0,client_width, client_height);
	// Draw the string
	g.drawString(title[currentIndex], titleX, titleY);
    }

    public void init() {

    }
    
    public void run() {
	//  Do our scrolling in here.
	//  First, change the scrolly value.  
	System.out.println("Running CreditsBox");
	while (true) {
	    if (scrolling) {
		scrolly += CREDIT_SCROLL_INCREMENT;  
		//  If scrolly is more than our height, make it zero,
		//  go to next string
		if (scrolly >= getSize().height) {
		    scrolly = 0;
		    //  Now get the next string
		    currentIndex++;
		    if (currentIndex >= title.length) {
			currentIndex = 0;
		    }
		}
		//  Now repaint the panel
		repaint();
	    }
	    //  Now sleep for a while
	    try {
		java.lang.Thread.sleep(100);
	    } catch (Exception uhoh) {
		//  Got some exception.  Don't do anything, we were 
		// just trying to sleep.
	    }
	}
    }
    
    public void setScroll(boolean s) {
	scrolling = s;
    }

    public void toggleScroll() {
	if (scrolling == true) {
	    scrolling = false;
	} else {
	    scrolling = true;
	}
    }

} // GMAOCreditsBox
