
/**
 * GMAOGUI.java
 *
 * This is the general driver component for the GMAO2k application.
 * It will handle opening and saving the XML files, and will allow the
 * user to pick a image to add as the next page of the manga. 
 * 
 * We also have to log the user in.  
 *
 * Created:
 * Thu Jun 22 10:45:59 2000
 *
 * @author David Evans
 * @version */

/* 
   GMAO - Image Annotation for Manga Translation
   Copyright (C) 2000  David Kirk Evans
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

   David Kirk Evans (devans@cs.columbia.edu)
   http://www.cs.columbia.edu/~devans/   
*/


// GMAO Stuff
import Annotator;
import XMLTreeDisplay;

// General java, swing stuff
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import org.w3c.dom.Document;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import java.util.*;

// Specific sun class we will use to write out the DOM
import com.sun.xml.tree.XmlDocument;


public class GMAOGUI extends JFrame implements ActionListener {
    JMenuBar menuBar;
    JFileChooser fc=null;
    Annotator currentAnnotator;
    File currentFile;
    JLabel annotatorInfo;
    JLabel fileInfo;
    Document mangaDoc;
    Properties prefs;  // Preferences in here
    XMLTreeDisplay xmlpanel; // Panel where XML tree for doc goes 
    XMLFileFilter xmlfilter; // File filter for xml docs
    ImageFileFilter imagefilter; // File filter for images
    GMAORemote server; // Remote update server
    GMAOAboutBox aboutBox; // Our about box
    AnnotationInformationPanel annInfoPanel;  // Handle to anninfopanel
    JFrame annInfoFrame; // Frame for above
    GMAOOptionsFrame optionsFrame;
    GMAOLanguagePairs languagePairs=null; // info on languages and annotators in current DOM

    public GMAOGUI(String title, Properties p) {
	super(title);
	prefs = p;
	
	// Set up the file chooser
	if (prefs.getProperty("applet") == null || !prefs.getProperty("applet").equals("true")) {
	    /* We can not initialize the file chooser if we are an applet 
	       due to a security violation.  Only do this if we are not 
	       an applet */
	    fc = new JFileChooser();
	}

	// Set up files filters for xml and images
	xmlfilter = new XMLFileFilter();
	imagefilter = new ImageFileFilter();
	
	/* Set up a menu for the gui */
	menuBar = new JMenuBar();
	setJMenuBar(menuBar);
	JMenu menu = new JMenu("File");
	menu.getAccessibleContext().setAccessibleDescription("File Menu");
	JMenuItem menuItem = new JMenuItem("New");
	menuItem.addActionListener(this);
	if (prefs.getProperty("applet") != null && prefs.getProperty("applet").equals("true")) {
	    // Disable this menu item
	    menuItem.setEnabled(false);
	}
	menu.add(menuItem);
	menuItem = new JMenuItem("Open");
	menuItem.addActionListener(this);
	if (prefs.getProperty("applet") != null && prefs.getProperty("applet").equals("true")) {
	    // Disable this menu item
	    menuItem.setEnabled(false);
	}
	menu.add(menuItem);
	menuItem = new JMenuItem("Save");
	menuItem.addActionListener(this);
	if (prefs.getProperty("applet") != null && prefs.getProperty("applet").equals("true")) {
	    // Disable this menu item
	    menuItem.setEnabled(false);
	}
	menu.add(menuItem);
	menuItem = new JMenuItem("Close");
	menuItem.addActionListener(this);
	if (prefs.getProperty("applet") != null && prefs.getProperty("applet").equals("true")) {
	    // Disable this menu item
	    menuItem.setEnabled(false);
	}
	menu.add(menuItem);
	menu.addSeparator();
	menuItem = new JMenuItem("Quit");
	menuItem.addActionListener(this);
	menu.add(menuItem);
	menuBar.add(menu);
	menu = new JMenu("Annotator");
	menu.getAccessibleContext().setAccessibleDescription("Operations concerning the Annotator");
	menuItem = new JMenuItem("Log in");
	menuItem.addActionListener(this);
	menu.add(menuItem);
	menuBar.add(menu);

	menu = new JMenu("GMAO");
	menuItem = new JMenuItem("New Page");
	menuItem.addActionListener(this);
	menu.add(menuItem);
	menuItem = new JMenuItem("New Chapter");
	menuItem.addActionListener(this);
	menu.add(menuItem);
	menu.addSeparator();

	/* Add some info to the panel */
	annotatorInfo = new JLabel("Annotator: Not logged in");
	fileInfo = new JLabel("File: No file selected");	

	/* Set up the layout to be a box layout */
	Box infoBox = new Box(BoxLayout.X_AXIS);
	infoBox.add(annotatorInfo);
	infoBox.add(infoBox.createHorizontalStrut(20));
	infoBox.add(fileInfo);
	
	/* Set the layout manager to be a BorderLayout, 
	   add the box to the North... */
	getContentPane().setLayout(new BorderLayout());
	getContentPane().add(infoBox, BorderLayout.NORTH);
	
	/* Add the XMLTreeDisplay to the center */
	xmlpanel = new XMLTreeDisplay(prefs);
	getContentPane().add(xmlpanel, BorderLayout.CENTER);
	
	//  Add an annotation information panel to a new frame
	annInfoFrame = new JFrame("Annotation Information Frame");
	annInfoPanel = new AnnotationInformationPanel(null, true, annInfoFrame, currentAnnotator);
	annInfoFrame.getContentPane().add(annInfoPanel);
	xmlpanel.setAnnInfoPanel(annInfoPanel);
	annInfoFrame.setSize(300,400);
	annInfoFrame.setVisible(true);

	// Should use actions, but instead I will use a simple hack.
	// Make a menu for the two things we want to do, and pass
	// the actionevent on to the annotationInformationPanel.
	menuItem = new JMenuItem("New Translation");
	menuItem.setMnemonic('n');
	menuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke("control N"));
	menuItem.addActionListener(this);
	menu.add(menuItem);
	menuItem = new JMenuItem("Edit Current Translation");
	menuItem.setMnemonic('e');
	menuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke("control E"));
	menuItem.addActionListener(this);
	menu.add(menuItem);
	menu.addSeparator();
	menuItem = new JMenuItem("Show Annotation Info Window");
	menuItem.addActionListener(this);
	menu.add(menuItem);
	menuItem = new JMenuItem("Options...");
	menuItem.addActionListener(this);
	menu.add(menuItem);
	menuItem = new JMenuItem("About GMAO...");
	menuItem.addActionListener(this);
	menu.add(menuItem);
	menuBar.add(menu);	
	AnnotatePanel.infoDisplay.setText("Please load an XML Manga file.");

	// Set up the about box
	aboutBox = new GMAOAboutBox();

	// Set up our options box
	optionsFrame = new GMAOOptionsFrame(prefs);
	
	/* Try to get a handle on the remote GMAOServer if we need to */
	if (prefs.getProperty("updateServer") != null) {
	    try {
		String serverString = "//"+prefs.getProperty("updateServer")+"/GMAOServer";
		server = (GMAORemote) java.rmi.Naming.lookup(serverString);
		xmlpanel.setServer(server);
		System.out.println("Got handle on "+serverString);
	    } catch (Exception e) {		
		System.out.println("Trouble trying to get a reference to remote server implementation!  Turning off remote updates!  : "+e);
		// Tell the user that they can't do remote updates
		JOptionPane.showMessageDialog(this, "Unable to connect to server "+prefs.getProperty("updateServer")+".  Updates will not be saved on the remote server.", "Server unreachable", JOptionPane.WARNING_MESSAGE);
		prefs.remove("updateServer");
	    }
	}
    }

    /**
     * updateDisplay()
     *
     * Put down a few labels that show information about the 
     * current Annotator and file, etc.
     */

    public void updateDisplay() {	
	/* Set up the labels for info display */
	if (currentAnnotator == null) {
	    annotatorInfo.setText("Annotator: Not logged in");
	} else {
	    annotatorInfo.setText("Annotator: "+currentAnnotator.getID()+
				  " ("+currentAnnotator.getName()+")");
	}
	if (currentFile == null) {
	    fileInfo.setText("File: No file selected");	
	} else {
	    fileInfo.setText("File: "+currentFile.getName());
	}
    }

    /*  Look for actions from our menu items */
    public void actionPerformed(ActionEvent e) {
	String command = e.getActionCommand();
	if (command.equals("Quit")) {
	    /*  Check for file saving, etc. */
	    if (prefs.getProperty("applet") != null && prefs.getProperty("applet").equals("true")) {
		// Don't want to throw an exception by trying to exit...
		annInfoFrame.hide();
		hide();
	    } else {
		// Save out properties...
		try {
		    FileOutputStream fos = new FileOutputStream("GMAO.props");
		    prefs.store(fos,"Great Manga Application Onidzuka Preferences.");
		    fos.close();
		} catch (Exception ex) {
		    System.out.println("Error trying to save GMAO.props..."+ex);
		}
		System.exit(0);
	    }
	} else if (command.equals("New")) {
	    System.out.println("New is not yet implemented");
	    /*  Should confirm that user wants to overwrite a file
	     *  if that is the case 
	     */
	    int returnVal = fc.showDialog(this, "New");
	    if (returnVal == JFileChooser.APPROVE_OPTION) {
		currentFile = fc.getSelectedFile();
		//this is where a real application would open the file.
	    } else {
		System.out.println("New command cancelled by user.");
	    }
	} else if (command.equals("Close")) {
	    System.out.println("Close is not yet implemented");
	} else if (command.equals("Save")) {
	    System.out.println("Save is implemented...poorly.");
	    //  Make sure we are saving the latest version of 
	    //  the mangaDoc.
	    xmlpanel.updateDom();
	    if (currentFile == null) {
		int returnVal = fc.showSaveDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
		    currentFile = fc.getSelectedFile();
		    //this is where a real application would save the file.
		    try {
			File temp = File.createTempFile("GMAO", ".xml", null);
			saveDom(temp);
			// If the save was successful, we can move the temp
			// file onto the real file.
			AnnotatePanel.infoDisplay.setText("Saved file "+currentFile.getName());			
		    } catch (java.io.IOException fe) {
			System.out.println("Error trying to create temp file/save: "+fe);
		    }		    
		} else {
		    System.out.println("Save command cancelled by user.");
		}
	    } else {
		/* We already have a File object in currentFile */
		saveDom(currentFile);
		AnnotatePanel.infoDisplay.setText("Saved file "+currentFile.getName());
	    }
	} else if (command.equals("Open")) {
	    System.out.println("Open is implemented...Poorly.");
	    // Let us only look for xml files
	    fc.resetChoosableFileFilters();
	    fc.addChoosableFileFilter(xmlfilter);
	    int returnVal = fc.showOpenDialog(this);	    
	    if (returnVal == JFileChooser.APPROVE_OPTION) {
		currentFile = fc.getSelectedFile();
		//this is where a real application would open the file.
		try {
		    openXMLFile((currentFile.toURL()).toExternalForm());
		} catch (java.net.MalformedURLException ex) {
		    System.out.println("Malformed URL: "+ex);
		    JOptionPane.showMessageDialog(this, "Error: "+currentFile+" isn't a valid URL!  No manga for you!", "Malformed URL", JOptionPane.ERROR_MESSAGE);
		} catch (java.io.UnsupportedEncodingException ioe) {
		    JOptionPane.showMessageDialog(this, "Fatal Error: This Java Environment doesn't seem to support japanese file encoding methods!\nLook for instructions on how to download i18n.jar.\nAlso, applet is now rendered totally useless.", "Jikusyo! Fatal Error!", JOptionPane.ERROR_MESSAGE);
		}

	    } else {
		System.out.println("Open command cancelled by user.");
	    }
	} else if (command.equals("Log in")) {
	    /* Log in the annotator.  Throw up a dialog and ask for
	       the name and id of the annotator */
	    JOptionPane jop = new JOptionPane();
	    String id = jop.showInputDialog(this, "Please enter the Annotator ID:");
	    String name = jop.showInputDialog(this, "Please enter the Annotator Name:");
	    // Should change this to be a password * type entry box but...
	    String pass = jop.showInputDialog(this, "Please enter your password:");
	    /** 
	     *  Check to see if we are doing remote updates.  If we are
	     *  then see if this Annotator can log in. 
	     */
	    if (prefs.getProperty("updateServer") != null) {
		// Check to see if we can log in
		try {
		    if (server.logInAnnotator(new Annotator(id, name, pass))) {
			// Log in successful!
			currentAnnotator = new Annotator(id, name, pass);
		    } else {
			// Not sucessful.  Either the names didn't match, 
			// or the passwords didn't match.  
			jop.showMessageDialog(this, "Your log in was denied.  Either the password did not match, or there is another user with this Annotator ID", "Log in denied!", JOptionPane.ERROR_MESSAGE);
		    }
		} catch (GMAOAnnotatorBannedException ba) {
		    jop.showMessageDialog(this, id+" or this machine's IP has been banned from annotating on "+prefs.getProperty("updateServer"), "Log in denied!", JOptionPane.ERROR_MESSAGE);
		} catch (java.rmi.RemoteException re) {
		    jop.showMessageDialog(this, "Unable to reach host: "+re);
		    System.out.println("RemoteException while trying to verify log in : "+re);
		}
	    } else if (id != null && name != null && pass != null) {
		currentAnnotator = new Annotator(id, name, pass);
	    }
	    xmlpanel.setAnnotator(currentAnnotator);
	} else if (command.equals("New Page")) {
	    // Add a new page to the mangaDoc
	    // Make sure we use the most up to date version of the DOM
	    xmlpanel.updateDom();
	    System.out.println("Updated dom in new page");
	    // Determine the name of the last image file, add 1 to it
	    // Get a list of all the pages in the document
	    org.w3c.dom.NodeList nodes = mangaDoc.getDocumentElement().getElementsByTagName("Page");	    
	    int last = nodes.getLength() - 1;
	    String imgName=""; // Current img name
	    int nextNum=0;  // Next page number
	    
	    if (last >= 0 && nodes.item(last).getNodeName().equals("Page")) {
		System.out.println("Looking at "+nodes.item(last));
		org.w3c.dom.NamedNodeMap attrs = nodes.item(last).getAttributes();
		imgName = attrs.getNamedItem("Image").getNodeValue();
		nextNum = Integer.parseInt(attrs.getNamedItem("Number").getNodeValue());
		nextNum++;
	    } else {
		System.out.println("The last child was not a Page node:");
		for (int j=0; j<nodes.getLength(); j++) {
		    System.out.println(nodes.item(j).getNodeName());
		}
	    }
	    // Find the number part of imgName, increment it by 1
	    System.out.println("About to get next image suggestion for ("+imgName+")");
	    String nextImg = getNextImageName(imgName);
	    if (nextImg != null) {
		System.out.println("New image for new page: "+nextImg);
		// Create the DOM node for the new page
		org.w3c.dom.Element newPage = mangaDoc.createElement("Page");
		System.out.println("Created new Page");
		// Set the Image attribute
		newPage.setAttribute("Image", nextImg);
		System.out.println("Set image attribute");
		// Set the number attribute
		newPage.setAttribute("Number", ""+nextNum);
		System.out.println("Set page number attribute");
		// Insert the page into the DOM		
		org.w3c.dom.NodeList chapters = mangaDoc.getDocumentElement().getElementsByTagName("Chapter");
		// Insert into last chapter
		if (chapters.getLength() == 0) {
		    System.out.println("Error: There are no chapters!  Please make a chapter first.");
		    AnnotatePanel.infoDisplay.setText("Please create a chapter before adding the first page.");
		} else {
		    chapters.item(chapters.getLength()-1).appendChild(newPage);
		    System.out.println("Inserted page into DOM");		    
		}
		// Set the new dom
		xmlpanel.setDOM(mangaDoc);
	    } else {
		// The user canceled the new image dialog.  
		// Might want to put some message somewhere... 
		AnnotatePanel.infoDisplay.setText("User cancelled image locate operation for new page command.");
	    }
	} else if (command.equals("New Chapter")) {
	    
	    // Make sure the DOM is up to date
	    xmlpanel.updateDom();
	    System.out.println("Updated DOM for new Chapter");
	    // Add a new Chapter.  Just make the new node in the DOM.
	    // The first node is the MangaTranslation, add under that.
	    org.w3c.dom.Element newChap = mangaDoc.createElement("Chapter");
	    // Add attribute "Number" to the new chapter -- get last chaper #
	    int num = getLastChapterNumber(mangaDoc);
	    newChap.setAttribute("Number", ""+num);
	    // Get the title for the chapter
	    String title = JOptionPane.showInputDialog(this, "Please enter the chapter title:");
	    if (title != null) {
		// Add a new node, ChapterTitle, with one element, the text title
		org.w3c.dom.Element chapTitle = mangaDoc.createElement("ChapterTitle");
		chapTitle.appendChild(mangaDoc.createTextNode(title));
		newChap.appendChild(chapTitle);
		/* Why is this lastchild?? You ask?  I think it is because 
		   the first node is actually a node that tells what type of
		   object the document is.  The second (and last) node is 
		   actually the first child, the MangaTranslation.
		*/
		mangaDoc.getLastChild().appendChild(newChap);
		// Alert the jtree that the model info changed 
		xmlpanel.setDOM(mangaDoc);
		// Try to update the remote server if we are supposed to
		if (server != null) {
		    try {
			//  Make the docID and send it over to xmlpanel.
			String docID = "";
			org.w3c.dom.NodeList tempL = mangaDoc.getDocumentElement().getElementsByTagName("SeriesName");
			docID += tempL.item(0).getFirstChild().getNodeValue();
			tempL = mangaDoc.getDocumentElement().getElementsByTagName("SeriesNumber");
			docID += tempL.item(0).getFirstChild().getNodeValue();
			server.addChapter(docID, num, title);
			JOptionPane.showMessageDialog(this, "Added new chapter on remote server", "New chapter saved", JOptionPane.INFORMATION_MESSAGE); 
		    } catch (java.rmi.RemoteException re) {
			JOptionPane.showMessageDialog(this, "Received remote exception from server: "+re, "Remote Exception", JOptionPane.ERROR_MESSAGE); 
			System.out.println("Remote exception while sending chapter update to server: "+re);
		    }
		}
	    }
	} else if (command.equals("About GMAO...")) {
	    // Display the about box
	    aboutBox.show();
	} else if (command.equals("New Translation")) {
	    // Send this action on to the AnnotationInformationPanel
	    annInfoPanel.actionPerformed(new ActionEvent(this, 8080, "New"));
	} else if (command.equals("Edit Current Translation")) {
	    // Send this action on to the AnnotationInformationPanel
	    annInfoPanel.actionPerformed(new ActionEvent(this, 8080, "Edit"));
	} else if (command.equals("Show Annotation Info Window")) {
	    annInfoFrame.show();
	} else if (command.equals("Options...")) {
	    optionsFrame.show();
	    // Now get the options
	    prefs = optionsFrame.getProperties();
	    // Send the props to the people that need it. 
	    xmlpanel.setProperties(prefs);
	} // the command types if block
	updateDisplay();
    }

    /**
     * getNextImageName()
     *  
     *  Given a string, return a new string that is the text preface of
     *  the string, followed by an incremented number portion. 
     *
     *  We do this in two major ways.  If we are using URLs to load the 
     *  image over http, we can't really do anything too complicated.
     *  We will just add a number to the name given in name, and put 
     *  up a dialog that lets the user approve the selection, or 
     *  type in a new one.  
     * 
     *  If we are using the local file system though, we will get a 
     *  directory listing, and get the next file after name.  Then we 
     *  will throw that up in a dialog, let the user change it, or
     *  give them a "browse" button that lets them use a JFileChooser to
     *  select the new file.  
     */
    String getNextImageName(String name) {
	if (name.equals("")) {
	    String ans = (String) JOptionPane.showInputDialog(this, "Please enter the image file name to get from "+prefs.getProperty("imageHttpLocation", "http://fugu.cs.columbia.edu/images/Gto_01/"), "Selecting image for new page...", JOptionPane.QUESTION_MESSAGE, null, null, "");
	    return ans;
	}
	int back = name.length()-5; // get rid of .ext also
	String letterSuffix="";
	// Check to see if we have a single character, like a,b,c,d
	if (Character.isLetter(name.charAt(back)) && Character.isDigit(name.charAt(back-1))) {
	    // We think we have something like 
	    // PRE_NUMx where x is a,b,c,d,etc
	    // If letter is a, make it b, if it is b, make a and increment
	    if (name.charAt(back) == 'a') {
		// Make letter b, return same file name
		String pref = name.substring(0, back);
		pref += "b";
		pref += name.substring(back+1, name.length());
		String ans = (String) JOptionPane.showInputDialog(this, "Please enter the image file name to get from "+prefs.getProperty("imageHttpLocation", "http://fugu.cs.columbia.edu/images/Gto_01/"), "Selecting image for new page...", JOptionPane.QUESTION_MESSAGE, null, null, pref);
		return ans;
	    } else if (name.charAt(back) == 'b') {
		letterSuffix = "a";
	    }
	    back = back-1;	    
	}
	int front = back;
	
	if (prefs.getProperty("imageSource", "http").equals("http")) {
	    // Go backwards until we see a char that isn't a number.
	    // Assumes that the structure is PRE_NUM.ext
	    while (name.charAt(front) >= '0' && name.charAt(front) <= '9') {
		front--;
	    }
	    // Now we have front pointing to the first number
	    // and back pointing to the last one.  
	    // Separate the string out.
	    String pre = name.substring(0, front+1);
	    int num = 0;
	    try {
		num = Integer.parseInt(name.substring(front+1, back+1));
	    } catch (NumberFormatException nfe) {
		String ans = (String) JOptionPane.showInputDialog(this, "Please enter the image file name to get from "+prefs.getProperty("imageHttpLocation", "http://fugu.cs.columbia.edu/images/Gto_01/"), "Selecting image for new page...", JOptionPane.QUESTION_MESSAGE, null, null, name);
		return ans;
	    }
	    if (name.charAt(back) != '.') {
		// If we had a single letter (a,b) skip over it. 
		// We just want to go from the '.' to the ext part
		back++;
	    }
	    String ext = name.substring(back+1, name.length());
	    System.out.println("Pre: "+pre+" Num: "+name.substring(front+1, back+1) +" Ext: "+ext);
	    // Increment the num, return new string.
	    num++;
	    //  We have to pad the number portion with the correct # of 0s
	    String ans = pre;
	    int numLength = new String(""+num).length();
	    for (int k=0; k<back-front-numLength-letterSuffix.length(); k++) {
		ans = ans + "0";  // Pad us with a 0
	    }
	    // add the rest of the string
	    ans = ans + num + letterSuffix + ext;
	    // Now, that is what we think it will look like. 
	    // We have to confirm with the user. 
	    ans = (String) JOptionPane.showInputDialog(this, "Please enter the image file name to get from "+prefs.getProperty("imageHttpLocation", "http://fugu.cs.columbia.edu/images/Gto_01/"), "Selecting image for new page...", JOptionPane.QUESTION_MESSAGE, null, null, ans);
	    return ans;
	} else {
	    // We are using the local file system.  
	    /*  We would like to get a listing of the directory
		and get the next image after imgName from it.
		Then we want to confirm this image with the
		user.  Guess what?  We don't want to let them 
		browse for an image, because all images must come
		from the same directory.  
	    */ 
	    //  Get listing of files
	    if (prefs.getProperty("imageFileLocation") == null) {
		JOptionPane.showMessageDialog(this, "Please set the imageFileLocation property in GMAO.prefs!", "Image file directory not set!", JOptionPane.ERROR_MESSAGE);
		return null;
	    }
	    File imageDir = new File(prefs.getProperty("imageFileLocation"));
	    String[] files = imageDir.list(imagefilter);
	    try {
		java.util.Arrays.sort(files);
	    } catch (ClassCastException e) {
		System.out.println("Error while trying to sort file array: "+e);
		return null;
	    }
	    // Have the sorted list of file names in files
	    // Find the one after name
	    int i=0;
	    while (i < files.length && ! files[i].equals(name)) {
		i++;
	    }
	    String ans;  // Put the tentative file name in here
	    if (i == files.length) {
		// Apparently, name is the last image in the directory
		ans = name;
	    } else {
	        ans = files[i+1];
	    }
	    //  Now, put up a dialog box to let the user confirm
	    ans = (String) JOptionPane.showInputDialog(this, "Please enter the image file name to get from "+prefs.getProperty("imageFileLocation"), "Selecting image for new page...", JOptionPane.QUESTION_MESSAGE, null, null, ans);
	    return ans;
	}
    }

    /**
     * scanDom(Document doc)
     *
     *  This method takes the dom and scans through all of its children.
     *  Right now it is looking for all "Character" attributes of all
     *  bubbles and inserting them into the TextCompleter.
     *
     *  Now we also look for all Annotation items, and track info on those 
     *  in our GMAOLanguagePairs languagePairs object.  
     */
    
    public void scanDom(Document doc) {
	// Should clear out text completer here, when I add that into textcompleter
	// Clear out the languagePairs 
	languagePairs = new GMAOLanguagePairs();
	
	// Get list of children
	org.w3c.dom.NodeList tops = doc.getLastChild().getChildNodes();
	// Find "Chapter" nodes, get list of their children, which are pages
	for (int k=0; k<tops.getLength(); k++) {
	    if (tops.item(k).getNodeName().equals("Chapter")) {
		org.w3c.dom.NodeList pages = tops.item(k).getChildNodes();
		// For each child (should be a page) get the bubbles
		int i=0;
		while (i<pages.getLength()) {
		    // Make sure it is a "Page"
		    if (pages.item(i).getNodeName().equals("Page")) {
			// Get the list of bubbles
			org.w3c.dom.NodeList bubbles = pages.item(i).getChildNodes();
			// Go through each bubble
			int j=0;
			while (j<bubbles.getLength()) {
			    // Make sure it is a bubble
			    if (bubbles.item(j).getNodeName().equals("Bubble")) {
				org.w3c.dom.NamedNodeMap attrs = bubbles.item(j).getAttributes();
				// Add the "Character" attribute to TextCompleter
				if (attrs.getNamedItem("Character") != null) {
				    TextCompleter.addText(attrs.getNamedItem("Character").getNodeValue());
				    AnnotatePanel.infoDisplay.setText("Scanned character "+ attrs.getNamedItem("Character").getNodeValue());
				}
				// Scan bubble children for Translation items, look for language annotator pair
				org.w3c.dom.NodeList translations = bubbles.item(j).getChildNodes();
				int deep=0;
				while (translations != null && deep<translations.getLength()) {
				    org.w3c.dom.NamedNodeMap tattrs = translations.item(deep).getAttributes();
				    // Add the info for the Language and Annotator - why did I call Annotator Author?...
				    if (tattrs != null && tattrs.getNamedItem("Language") != null && tattrs.getNamedItem("Author") != null) {
					languagePairs.incrementAnnotationCount(tattrs.getNamedItem("Author").getNodeValue(), tattrs.getNamedItem("Language").getNodeValue());
				    }
				    deep++;
				}
			    }
			    j++;
			}
		    }
		    i++;
		}
	    }
	}
	// TextCompleter.debugInfo();
	AnnotatePanel.infoDisplay.setText("Scanned DOM for Characters...");
	// Tell the options frame about the new language pairs
	optionsFrame.setLanguagePairs(languagePairs);
	languagePairs.debugInfo();
    }

    /**
     * getLastChapterNumber(Document doc) 
     *
     * Look at the Document doc, and spin through it until we see the
     * last "Chapter" node.  Return the Number of that chapter.
     */
    int getLastChapterNumber(Document doc) {
	org.w3c.dom.NodeList nodes = doc.getDocumentElement().getElementsByTagName("Chapter");
	int last = nodes.getLength() - 1;
	// The last child is in last now - get the chapter number from it.
	int ans=0;
	if (nodes.item(last).getNodeName().equals("Chapter")) {
	    org.w3c.dom.NamedNodeMap attrs = nodes.item(last).getAttributes();
	    ans = Integer.parseInt(attrs.getNamedItem("Number").getNodeValue());
	    ans++;
	} else {
	    System.out.println("The last child was not a Chapter node!!");
	}
	return ans;
    }    

    /**
     * openXMLFile(String theURI)
     *
     *  The URI is a Universal Resource locator.  File, http, etc.
     * 
     *  Use the JAXP API to open up the XML file and parse it into a DOM.
     *  Rough first pass attempt. 
     *  
     *  Also sets the DocID up for this document, and passes it on to
     *  the xmltree panel that needs it to do remote updates.
     */
    public void openXMLFile(String theURI) throws java.io.UnsupportedEncodingException {

	try {
	    DocumentBuilderFactory 
		docBuilderFactory = DocumentBuilderFactory.newInstance();
	    DocumentBuilder 
		docBuilder = docBuilderFactory.newDocumentBuilder();
	    
	    mangaDoc = docBuilder.parse(theURI);
	    
	    if (mangaDoc == null) {
		AnnotatePanel.infoDisplay.setText("Your JVM does not support internalationization.  Please install i18n.jar onto your $CLASSPATH");
	    }

	    // normalize text representation
	    mangaDoc.getDocumentElement().normalize();

	    // Now we should scan the DOM for characters to complete...
	    scanDom(mangaDoc);
	    //  We now have the XML document in MangaDoc.  We should
	    //  display its tree in the center panel.  
	    xmlpanel.setDOM(mangaDoc);
	    AnnotatePanel.infoDisplay.setText("Loaded file "+theURI);
	    
	    //  Make the docID and send it over to xmlpanel.
	    String docID = "";
	    org.w3c.dom.NodeList tempL = mangaDoc.getDocumentElement().getElementsByTagName("SeriesName");
	    docID += tempL.item(0).getFirstChild().getNodeValue();
	    tempL = mangaDoc.getDocumentElement().getElementsByTagName("SeriesNumber");
	    docID += tempL.item(0).getFirstChild().getNodeValue();
	    xmlpanel.setDocID(docID);
	} catch (SAXParseException err) {
	    JOptionPane.showMessageDialog(this, "Error parsing XML file: "+err.getMessage(), "XML Parsing error", JOptionPane.ERROR_MESSAGE); 
	    System.out.println ("** Parsing error" 
				+ ", line " + err.getLineNumber ()
				+ ", uri " + err.getSystemId ());
	    System.out.println("   " + err.getMessage ());
	    // print stack trace as below
	} catch (SAXException e) {
	    JOptionPane.showMessageDialog(this, "SAX Error while parsing XML file", "XML Parsing error", JOptionPane.ERROR_MESSAGE); 
	    Exception	x = e.getException ();

	    ((x == null) ? e : x).printStackTrace ();
	} catch (java.io.UnsupportedEncodingException ioe) {
	    // Throw it up out
	    throw ioe;
	} catch (Throwable t) {
	    t.printStackTrace ();
	}
    }
    
    /**
     * void saveDom(File theFile) 
     *
     *  Write out the dom in mangaDoc to the file theFile.
     *  This is using code from the JAXP tutorial, and relies on the
     *  Sun classes...  So it will have to change if the underlying
     *  implementation changes.  
     */  
    public void saveDom(File theFile) {
	// Cast our mangaDoc to a specific sun implementation.
	XmlDocument xdoc = (XmlDocument) mangaDoc;
	// Write it out to a File
	try {
	    FileOutputStream fos = new FileOutputStream(theFile);
	    xdoc.write(new OutputStreamWriter(fos, "euc-jp"));
	    // Ensure that the file is closed.
	    fos.close();
	} catch (java.io.IOException e) {
	    System.out.println("Couldn't write out the DOM, file io error: "+e);
	}
    }

    public static void main(String s[]) {
	// Try to load in a preferences file
	Properties myprefs = new Properties();
	try {
	    myprefs.load(new FileInputStream("GMAO.props"));
	    System.out.println("Loaded in GMAO.props");
	} catch (FileNotFoundException e) {
	    System.out.println("GMAO.props does not exist, using defaults.");
	} catch (IOException e) {
	    System.out.println("Error loading GMAO.props: "+e);	    
	}
	
        GMAOGUI gui = new GMAOGUI("GMAO2k", myprefs);
	
        gui.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {System.exit(0);}
        });

	/* Add a menu to the frame here */

        gui.pack();
        gui.setVisible(true);

	/* Check to see if we have a XML file specified.  If we do,
	   load the document straight away.
	*/
	if (myprefs.getProperty("xmlFile") != null) {
	    AnnotatePanel.infoDisplay.setText("Auto-loading "+myprefs.getProperty("xmlFile")+" please wait...");
	    try {
		gui.openXMLFile(myprefs.getProperty("xmlFile"));
	    } catch (java.io.UnsupportedEncodingException ioe) {
		JOptionPane.showMessageDialog(gui, "Fatal Error: This Java Environment doesn't seem to support japanese file encoding methods!  Look for instructions on how to download i18n.jar...  Also, applet is now rendered totally useless.", "Jikusyo! Fatal Error!", JOptionPane.ERROR_MESSAGE);
	    }
	}
    }

} // GMAOGUI
