
/**
 * GMAORemote.java
 *
 *  This is the remote interface that defines the communication 
 *  between a GMAO Client (GMAOGUI) and the GMAOServer.  
 *
 * Created: Mon Jul 24 08:27:36 2000
 *
 * @author devans@cs.columbia.edu
 * @version 0.1b
 */


/* 
   GMAO - Image Annotation for Manga Translation
   Copyright (C) 2000  David Kirk Evans
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

   David Kirk Evans (devans@cs.columbia.edu)
   http://www.cs.columbia.edu/~devans/   
*/

import java.rmi.Remote;
import java.rmi.RemoteException;
import org.w3c.dom.*;
import java.util.Vector;

public interface GMAORemote extends Remote {
    /**
     *  This method logs an Annotator in.  If the Annotator already
     *  exists, and the password does not match, then false is
     *  returned.  If the Annotator exists, and the password matches,
     *  true is returned.  Or if the Annotator does not exist, a new
     *  one is created and their password is recorded.  
     *
     *  Throws GMAOAnnotatorBannedException if the Annotator has been
     *  banned on this server.
     */
    public boolean logInAnnotator(Annotator a) throws RemoteException, GMAOAnnotatorBannedException;
    /**
     *  This method updates the DOM in the remote server.
     * 
     *  Initially, I was going to just pass a DocumentFragment, but
     *  hey, the sun implementation of them isn't serializable.  Since
     *  it isn't required, other parses might use serializable
     *  classes, or they might not.  Anyway, that gave me the
     *  motivation to change the system around so that I just track
     *  instead annotations that have changed, and then I will send a
     *  vector of only the changed annotations.  Guess what?  Thanks
     *  to non-serializability of Shape, that won't work either.  I am
     *  now down to plan-C: change the docfragment of the changes into
     *  a string, send that, and re-parse it.  Ugly.  But, possilbe.
     *  Use a built in DTD just for this little page thinger.
     *
     *  The docID string identifies the XML document that we are
     *  working on.  It is a combination of the XML fields series
     *  title and series number.  
     *
     *  All changes that are made go to the coressponging page by page
     *  number in the remote dom.  So the page numbers had better be
     *  synced.  There are also some problems where if one person adds
     *  more bubbles while another person is using an older copy, then
     *  badness can happen...  
     */
    public void updateDOM(String docID, String changedXMLString) throws RemoteException;

    /**
     *  This method is used to indicate that a new chapter has been 
     *  added to the manga.  All subsequent added pages will go into 
     *  this chapter.
     */
    public void addChapter(String docID, int number, String title) throws RemoteException;
} // GMAORemote



