
/**
 * NoteCustomField.java
 *
 * This class implements information pertinant to a custom field
 * defined for a Note, which is part of an Annotation.  The custom
 * fields are fairly simple objects, they track the (text) name of the
 * field, the (text) value of the field, and a boolean value that
 * indicates whether this field is required for every note, or is
 * optional.
 *
 * Created: Tue Mar 28 21:15:00 2000
 *
 * @author David Evans
 * @version 0.1a */

/* 
   GMAO - Image Annotation for Manga Translation
   Copyright (C) 2000  David Kirk Evans
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

   David Kirk Evans (devans@cs.columbia.edu)
   http://www.cs.columbia.edu/~devans/   
*/


public class NoteCustomField implements java.io.Serializable {
    boolean required = true;
    String fieldName;
    String fieldValue;
    
    /**
     *  Create a new NoteCustomField that has a field name of name, a
     *  value of value, and req is the required status of this custom
     *  field.  
     */
    
    public NoteCustomField(String name, String value, boolean req) {
	required = req;
	fieldName = name;
	fieldValue = value;
    }
    
    // Accessor functions
    public void setValue(String val) {
	fieldValue = val;
    }
    public void setName(String name) {
	fieldName = name;
    }
    public void setRequired(boolean req) {
	required = req;
    }
    public String getValue() {
	return fieldValue;
    }
    public String getName() {
	return fieldName;
    }
    public boolean isRequired() {
	return required;
    }
} // NoteCustomField
