
/**
 * PercentComboBoxEditor.java
 *
 *  This is a ComboBoxEditor that uses a PercentDocument as the Document
 *  model for the JTextField that this editor uses.  This means that we
 *  will only be able to type in numbers for the JComboBox that uses
 *  this editor.  
 *
 * Created: Wed Jul 19 11:36:28 2000
 *
 * @author David Evans
 * @version
 */


/* 
   GMAO - Image Annotation for Manga Translation
   Copyright (C) 2000  David Kirk Evans
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

   David Kirk Evans (devans@cs.columbia.edu)
   http://www.cs.columbia.edu/~devans/   
*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.JTextField;
import javax.swing.ComboBoxEditor;

public class PercentComboBoxEditor implements ComboBoxEditor, ActionListener {
    private JTextField edComponent;
    private javax.swing.event.EventListenerList listenerList;
    private ActionEvent actionEvent=null;

    public PercentComboBoxEditor() {
	listenerList = new javax.swing.event.EventListenerList();
	edComponent = new JTextField(new PercentDocument(), "100%", 10);
	edComponent.addActionListener(this);
    }
    
    /**
     *  return the instance of a JTextField with a PercentDocument 
     *  that we use for editing.
     */
    public java.awt.Component getEditorComponent() {
	return ((java.awt.Component) edComponent);
    }
    
    /**
     * get the String from edComponent that has been edited
     */
    public Object getItem() {
	return edComponent.getText();
    }

    /**
     * setItem sets the item that should be edited in the editor
     */
    public void setItem(Object anItem) {
	edComponent.setText((String)anItem);
    }

    /**
     * selectAll selects all the text in the editor
     */
    public void selectAll() {
	edComponent.selectAll();
    }

    /**
     *  add an action listener
     */

    public void addActionListener(ActionListener l) {
	listenerList.add(ActionListener.class, l);
    }

    /**
     * remove an action listener
     */

    public void removeActionListener(ActionListener l) {
	listenerList.remove(ActionListener.class, l);
    }

    /**
     * here we listen for our editor Component's action events
     * and tell our ActionListeners that the event was fired
     */
    public void actionPerformed(ActionEvent e) {
	// Guaranteed to return a non-null array
	Object[] listeners = listenerList.getListenerList();
	// Process the listeners last to first, notifying
	// those that are interested in this event
	for (int i = listeners.length-2; i>=0; i-=2) {
	    if (listeners[i]==ActionListener.class) {
		// Lazily create the event:
		if (actionEvent == null)
		    actionEvent = new ActionEvent(this, 1, "comboBoxChanged");
		((ActionListener)listeners[i+1]).actionPerformed(actionEvent);
	    }	       
	}
    }

} // PercentComboBoxEditor





