
/**
 * TextCompleter.java
 *
 * Created: Sun Jul 09 17:42:58 2000
 *
 *  This class is used to predict text completions ala alt-tab in emacs. 
 *  Anyway, in general, here is the idea.  We will make all methods and 
 *  variables in this class static, because I really want to have this
 *  global (bad I know.  But I am not going to pass around references 
 *  10 levels deep in GMAO to deal with this.  Anyway, we will tool 
 *  through the DOM when we first open it, and insert each character
 *  into the TextCompleter.  Then it knows what names have been seen
 *  before.  When we are passed a string, we will look to see what 
 *  text we have that it could complete to.  Simple, eh?  
 *
 *  At a later date, I would like to add information on how many times
 *  each text string has been seen.  This would allow us to return
 *  higher frequency strings first.  
 *
 * @author David Evans
 * @version 0.1b
 */

/* 
   GMAO - Image Annotation for Manga Translation
   Copyright (C) 2000  David Kirk Evans
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

   David Kirk Evans (devans@cs.columbia.edu)
   http://www.cs.columbia.edu/~devans/   
*/

import java.util.Vector;

public class TextCompleter  {
    static Vector text = new Vector(); // Keep strings in here, sorted!  
    
    public TextCompleter() {
	
    }

    public static void addText(String t) {
	// First, see if we have the string already
	if (exists(t)) {
	    // If we add frequency info, it goes here
	    return;
	} else {
	    //  Need to add the text in.  Search for it first. 
	    int i=0;
	    while (i < text.size() && ((String)text.elementAt(i)).compareTo(t) < 0) {
		i++;
	    }
	    //  We can insert the string here now.
	    text.insertElementAt(t, i);
	}
    }
    
    public static boolean exists(String t) {
	int i=0;
	while (i < text.size() && ((String)text.elementAt(i)).compareTo(t) < 0) {
	    i++;
	}
	// Check if we checked the whole vector
	if (i == text.size()) {
	    return false;
	}
	// Check if we are the same as the string at i
	if (((String)text.elementAt(i)).compareTo(t) == 0) {
	    return true;
	}
	return false;
    }

    /**
     * public static String predict(String t)
     *
     *  This function will search all of the text that has been added
     *  via addText, and returns a String if there is an unambigious 
     *  string that can be completed.
     *
     *  In the case that there are multiple possible compeletions, we 
     *  will return the string that completes the text up to the point
     *  where it becomes ambigious.  
     */
    public static String predict(String t) {
	int i=0;
	// Loop conditions are tricky.  Have to make sure we don't do a 
	// substring longer than the string itself...!
	while (i < text.size() && ((String)text.elementAt(i)).substring(0, t.length() > ((String)text.elementAt(i)).length() ? ((String)text.elementAt(i)).length() : t.length()).compareTo(t) < 0) {
	    i++;
	}
	// Now, we either went through the whole array
	if (i == text.size()) {
	    // System.out.println("Didn't find anything in Vector!");
	    return null;
	} else if (((String)text.elementAt(i)).substring(0, t.length()  > ((String)text.elementAt(i)).length() ? ((String)text.elementAt(i)).length() : t.length()).compareTo(t) > 0) {
	    // System.out.println("Quit because " + (String)text.elementAt(i) + " > " + t);
	    // Found stuff greater than t
	    return null;
	} else {
	    // we found something that has the same first characters as t.
	    // I guess we will just return the String if there is only
	    // one string that matches.  
	    // If this is the last item in the vector, we can return it.
	    // System.out.println("Looking at things like "+t);
	    if (i + 1 == text.size()) {
		return (String)text.elementAt(i);
	    }
	    // Check to see if there is another string after i that is the same
	    if (((String)text.elementAt(i+1)).substring(0, t.length()  > ((String)text.elementAt(i+1)).length() ? ((String)text.elementAt(i+1)).length() : t.length()).compareTo(t) == 0) {
		// The string after the one we found is the same.  
		// Collect the set of strings that have this same prefix.
		Vector matches = new Vector();
		while (i < text.size() && ((String)text.elementAt(i)).substring(0, t.length()  > ((String)text.elementAt(i)).length() ? ((String)text.elementAt(i)).length() : t.length()).compareTo(t) == 0) {
		    // If the string we are looking at is longer than t
		    if (((String)text.elementAt(i)).length() > t.length()) {
			// Add this String to a Vector		    
			matches.add((String)text.elementAt(i));
		    }
		    i++;
		}
		// Now we have a Vector of matches.  Return the longest
		// common prefix
		String lcp = longestCommonPrefix(matches);
		if (lcp.length() < t.length()) {
		    // The unambigious text is less than the text entered
		    return null;
		} else {
		    return lcp;
		}
	    } else {
		// We can return the unambigious completed text
		return (String)text.elementAt(i);
	    }
	}
    }           

    /**
     *  debugInfo
     *  print out some debugging info to System.out
     */
    static public void debugInfo() {
	System.out.println("Text in TextCompleter:");
	int i=0;
	while (i < text.size()) {
	    System.out.println((String)text.elementAt(i));
	    i++;
	}
    }

    /**
     *  String longestCommonPrefix(Vector strings)
     * 
     *  Return the longest common prefix that all the strings in strings
     *  share.  "" if there is no common prefix.
     */
    public static String longestCommonPrefix(Vector strings) {
	// Look at each letter one at a time, building up lcp
	String lcp = "";
	if (strings.size() == 0) {
	    return lcp;
	}
	int charIndex=0; // Index for char we are looking at
	int shortestString = shortestString(strings);
	// At most look up to the length of the shortest string
	while (charIndex < shortestString) {
	    // Pick up the char at charIndex in the first string
	    // Look at each string, if it has the same char we are ok.
	    for (int i=0; i<strings.size(); i++) {
		if (((String)strings.elementAt(0)).charAt(charIndex) != ((String)strings.elementAt(i)).charAt(charIndex)) {
		    // The characters were not equal.  Return lcp so far.
		    return lcp;
		}
	    }
	    // Add the character to the lcp, increase index
	    lcp += ((String)strings.elementAt(0)).charAt(charIndex);
	    charIndex++;
	}
	return lcp;
    }
    
    /**
     *  int shortestString(Vector strings) 
     *  
     *  Return the length of the shortest string in the vector strings
     */
    public static int shortestString(Vector strings) {
	int shortest = Integer.MAX_VALUE;
	if (strings.size() == 0) {
	    // Shouldn't happen
	    return 0;
	}
	for (int i=0; i<strings.size(); i++) {
	    if (((String)strings.elementAt(i)).length() < shortest) {
		shortest = ((String)strings.elementAt(i)).length();
	    }
	}
	return shortest;
    }
} // TextCompleter




