
/**
 * TextCompletionDocument.java
 *
 *  This class is an extension of the swing PlainDocument class.  
 *  It calls TextCompleter.predict(String t) to predict what 
 *  the field contents should be.  If TextCompleter returns a 
 *  completion, the document is changed to have that completion. 
 *  Otherwise, the text is accepted as it is typed in.  
 *
 * Created: Sun Jul 09 22:43:34 2000
 *
 * @author David Evans
 * @version
 */

/* 
   GMAO - Image Annotation for Manga Translation
   Copyright (C) 2000  David Kirk Evans
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

   David Kirk Evans (devans@cs.columbia.edu)
   http://www.cs.columbia.edu/~devans/   
*/

import javax.swing.text.*;

public class TextCompletionDocument extends PlainDocument {
    
    public TextCompletionDocument() {
	super();
    }

    public void insertString(int offs, String str, AttributeSet a)
                    throws BadLocationException {
	// Get the text from the start up to where our delta is
	String text = super.getText(0, offs);
	// Append our delta, see if it is in TextCompleter
	String prefix = text + str;
	String predict = TextCompleter.predict(prefix);
	// System.out.println("Prefix: "+prefix+"\tPred: "+predict);
	if (predict == null) {
	    // No prediction, just add what we got like normal
	    super.insertString(offs, str, a);
	} else {
	    // Got a prediction.  
	    // Add in the text at offs that is predict from offs to end
	    super.insertString(offs, predict.substring(offs, predict.length()), a);
	}
    }
    
} // TextCompletionDocument
